
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

DevConsole = {}
DevConsole.__index = DevConsole

-- =======================================================================

function DevConsole.new (options)

	local new = {}
	setmetatable(new, DevConsole)

	new.lines = Collection.new()
	new.font = love.graphics.newFont('assets/fonts/Exo2-Medium.otf', 12)
	new.inputLine = DevConsoleLine.new(new, DevConsoleLineKind.CurrentInput)
	new.inputLine:SetPrefix('> ')
	new.input = ''
	new.commands = Map.new()
	new.vars = Map.new()
	new.maxLines = 8
	new.skipLines = 0
	new.lineSizeY = 16

	return new

end

-- lines =================================================================
-- =======================================================================

function DevConsole:PushLine (kind, text)

	local line = DevConsoleLine.new(self, kind, text)
	self.lines:Add(line)

	if #self.lines > self.maxLines then
		self.skipLines = self.skipLines + 1
	end

end

-- interpreter ===========================================================
-- =======================================================================

function DevConsole:Execute (line)

	if #line:trim() == 0 then
		return
	end

	-- Set variable value
	local tokens = line:explode('=')
	for n = 1, #tokens do

		tokens[n] = tokens[n]:trim()

	end

	if #tokens == 2 then

		local var = self.vars[tokens[1]]
		if not var then

			self:PushLine(DevConsoleLineKind.ConsoleError, strFormat("Unknown variable '%s'.", tokens[1]))
			return

		end

		var:SetValue(tokens[2])
		self:PushLine(DevConsoleLineKind.Output, strFormat("'%s' variable set to %s", tokens[1], toStr(var:GetValue())))
		return

	end

	-- Execute command
	local tokens = line:explode(' ')
	local command = self.commands[tokens[1]]
	if command then

		table.remove(tokens, 1)

		for n, token in ipairs(tokens) do
			tokens[n] = tonumber(token) or token
		end

		command:Execute(unpack(tokens))
		return

	end

	-- Show variable value
	if #tokens == 1 then

		local var = self.vars[line]
		if var then

			self:PushLine(DevConsoleLineKind.Output, strFormat("%s: %s", line, toStr(var:GetValue())))
			return

		end

	end

	self:PushLine(DevConsoleLineKind.ConsoleError, strFormat("Unknown command/variable '%s'", tokens[1]))

end

-- =======================================================================

function DevConsole:ExecuteFile (sourceFilePath)
	
	local file = InputFile.new(sourceFilePath)
	for line in file:EachLine() do

		if not line:startsWith('#') then

			self:Execute(line)

		end

	end

	file:Close()

end

-- vars ==================================================================
-- =======================================================================

function DevConsole:LearnVar (name, value)

	local var = DevConsoleVar.new(name, value)

	assert(not self.vars:ContainsKey(name), strFormat("Duplicate development console variable name '%s'.", name))
	self.vars[name] = var

end

-- =======================================================================

function DevConsole:GetVar (name)
	
	return assert(self.vars[name], strFormat("Dev console variable '%s' doesn't exist.", name))

end

-- commands ==============================================================
-- =======================================================================

function DevConsole:LearnCommand (name, action)

	local command = DevConsoleCommand.new(self, name, action)

	assert(not self.commands:ContainsKey(name), strFormat("Duplicate development console command name '%s'.", name))
	self.commands[name] = command

end

-- autocomplete ==========================================================
-- =======================================================================

function DevConsole:Autocomplete (listPossibilities)

	local line = self.input

	local possibilities = Collection.new()
	local isPossible = function (text) return text:startsWith(line) end

	possibilities:Append(self.commands:ListKeys():Select(isPossible))
	possibilities:Append(self.vars:ListKeys():Select(isPossible))

	if #possibilities == 0 then
		return
	end

	if #possibilities == 1 then

		self:SetInput(possibilities[1] .. ' ')
		return

	end

	local len = #line
	local continue = true
	while continue do

		len = len + 1
		local str = possibilities[1]:sub(1, len)
		for p in possibilities:Each() do

			if #p < len then

				continue = nil
				break

			end

			if not p:startsWith(str) then

				continue = nil
				break

			end

		end

	end

	local common = possibilities[1]:sub(1, len - 1)
	self:SetInput(common)

	if listPossibilities then

		for p in possibilities:Sort():Each() do
			self:PushLine(DevConsoleLineKind.AutocompleteHint, p)
		end

	end

end

-- draw ==================================================================
-- =======================================================================

function DevConsole:Draw()

	if not self.visible then 
		return 
	end

	self:DrawBack()
	self:DrawLines()

end

-- =======================================================================

function DevConsole:DrawBack()

	local lineCount = math.min(#self.lines, self.maxLines) + 1
	local backSizeY = lineCount * self.lineSizeY

	love.graphics.setColor(32, 32, 72, 232)
	love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), backSizeY)

end

-- =======================================================================

function DevConsole:DrawLines()

	local index
	for n = 1, self.maxLines do

		index = n
		local line = self.lines[self.skipLines+index]
		if not line then
			index = index - 1
			break
		end

		local screenY = (index-1) * self.lineSizeY
		line:Draw(0, screenY)

	end

	local screenY = index * self.lineSizeY
	self.inputLine:Draw(0, screenY)

end

-- recall ================================================================
-- =======================================================================

-- The 'recallIndex' points at the last line that we used as an input to recall.

function DevConsole:RecallBack()

	local index = self.recallIndex or #self.lines+1
	for n = index-1, 1, -1 do

		if self.lines[n]:GetKind() == DevConsoleLineKind.Input then
			self:RecallLine(n)
			return
		end

	end

end

-- =======================================================================

function DevConsole:RecallForward()

	local index = self.recallIndex or 1
	for n = index+1, #self.lines do

		if self.lines[n]:GetKind() == DevConsoleLineKind.Input then
			self:RecallLine(n)
			return
		end

	end

end

-- =======================================================================

function DevConsole:RecallLine (index)

	self.recallIndex = index
	self:SetInput(self.lines[index]:GetText())

end

-- input =================================================================
-- =======================================================================

function DevConsole:OnKeyPress (key)

	if key == 'backspace' then
		
		self:SetInput(self.input:sub(1, -2))
	
	elseif key == 'return' or key == 'kpenter' then

		if #self.input > 0 then

			self:PushLine(DevConsoleLineKind.Input, self.input)
			self:Execute(self.input)
			self.recallIndex = nil
			self:SetInput('')

		end

	elseif key == 'tab' then

		local listPossibilities = love.keyboard.isDown('lctrl')
		self:Autocomplete(listPossibilities)

	elseif key == 'up' then

		self:RecallBack()

	elseif key == 'down' then

		self:RecallForward()

	end

end

-- =======================================================================

function DevConsole:OnTextInput (text)

	if not self:IsVisible() then
		return
	end

	self:SetInput(self.input .. text)

end

-- =======================================================================

function DevConsole:SetInput (input)

	self.input = input
	self.inputLine:SetText(input)

end

-- visibility ============================================================
-- =======================================================================

function DevConsole:Show()

	self.visible = true

end

-- =======================================================================

function DevConsole:Hide()

	self.visible = nil

end

-- =======================================================================

function DevConsole:IsVisible()

	return self.visible

end

-- scroll ================================================================
-- =======================================================================

function DevConsole:ScrollBack (lines)

	self.skipLines = math.max(self.skipLines - lines, 0)

end

-- =======================================================================

function DevConsole:ScrollForward()

	self.skipLines = math.min(self.skipLines + lines, self.maxLines - #self.lines)

end

-- font ==================================================================
-- =======================================================================

function DevConsole:GetFont()

	return self.font

end

