
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Deck = {}
Deck.__index = Deck

-- =======================================================================

function Deck.new()

	local new = {}
	setmetatable(new, Deck)
	
	new.modules = Slots.new()
	new.chips = Slots.new()
	new.detections = Map.new()
	new.fuses = Collection.new()

	return new

end

-- CONSOLE ===============================================================
-- =======================================================================

function Deck:SetConsole (console)
	
	self.console = consoleEntities:Get(console)

	self.modules:SetCount(self.console:GetModuleSlots())
	self.chips:SetCount(self.console:GetChipSlots())

end

-- KERNEL ================================================================
-- =======================================================================

function Deck:SetKernel (kernel)
	
	self.kernel = kernel

end

-- ... ===================================================================
-- =======================================================================

function Deck:Rebuild()
	
	self:RebuildVision()
	self:RebuildTokens()
	self:RebuildTokensPerAccess()
	self:RebuildTraceSpeedMod()
	self:RebuildTraceSpeedModPerAlert()
	self:RebuildDetections()
	self:RebuildDownloadSpeed()
	self:RebuildDownloadUnderTrace()
	self:RebuildFuses()

end

-- =======================================================================

function Deck:RebuildVision()
	
	self.vision = Map.new()
	self.vision[NodeVisibility.Layout] = 2
	self.vision[NodeVisibility.Class] = 1

	for aspect in self:GetAspectsWithTrait(DeckTrait.Vision):Each() do

		for vision, extension in aspect:GetValues():Each() do

			local vision = NodeVisibility:FromStr(vision)
			self.vision[vision] = self.vision[vision] + extension

		end

	end

end
	
-- =======================================================================

function Deck:RebuildTokens()

	self.maxTokens = self:GetKernelInstance():GetMaxTokens()
	self.startTokens = self:GetKernelInstance():GetStartTokens()

	for aspect in self:GetAspectsWithTrait(DeckTrait.StartTokens):Each() do

		self.startTokens = self.startTokens + aspect:GetValues()['tokens']

	end

end

-- =======================================================================

function Deck:RebuildTokensPerAccess()
	
	self.tokensPerAccess = {}
	for aspect in self:GetAspectsWithTrait(DeckTrait.TokensPerAccess):Each() do

		local values = aspect:GetValues()
		local class = NodeClass:FromStr(values['class'])
		
		self.tokensPerAccess[class] = (self.tokensPerAccess[class] or 0) + values['tokens']

	end

end

-- =======================================================================

function Deck:RebuildTraceSpeedMod()
	
	self.traceSpeedMod = 1
	for aspect in self:GetAspectsWithTrait(DeckTrait.SlowTrace):Each() do

		self.traceSpeedMod = self.traceSpeedMod * aspect:GetValues()['mod']

	end

end

-- =======================================================================

function Deck:RebuildTraceSpeedModPerAlert()
	
	self.traceSpeedModPerAlert = 0.5
	for aspect in self:GetAspectsWithTrait(DeckTrait.TraceSpeedPerAlertModReduction):Each() do

		self.traceSpeedModPerAlert = math.min(self.traceSpeedModPerAlert, aspect:GetValues()['mod'])

	end

end

-- =======================================================================

function Deck:RebuildDetections()
	
	self.detections = Map.new()
	for aspect in self:GetAspectsWithTrait(DeckTrait.Detection):Each() do

		local trait = SubroutineTrait:FromStr(aspect:GetValues()['trait'])
		self.detections[trait] = true

	end

end

-- =======================================================================

function Deck:RebuildDownloadSpeed()
	
	self.downloadSpeedMod = 1
	for aspect in self:GetAspectsWithTrait(DeckTrait.FastDownload):Each() do

		self.downloadSpeedMod = self.downloadSpeedMod * aspect:GetValues()['mod']

	end

end

-- =======================================================================

function Deck:RebuildDownloadUnderTrace()
	
	self.downloadUnderTrace = nil
	for aspect in self:GetAspectsWithTrait(DeckTrait.DownloadUnderTrace):Each() do

		self.downloadUnderTrace = true

	end

end

-- =======================================================================

function Deck:RebuildFuses()
	
	self.fuses:Clear()
	for _, chip in self.chips:EachNotBlank() do

		local chip = chipEntities:Get(chip)
		for fuse in chip:GetFuses():Each() do

			self.fuses:Add(fuse)

		end

	end

end

-- ... ===================================================================
-- =======================================================================

function Deck:GetAspectsWithTrait (trait)

	local result = Collection.new()
	for _, chip in self.chips:EachNotBlank() do

		local chip = chipEntities:Get(chip)
		for aspect in chip:GetAspects():Each() do

			if aspect:GetTrait() == trait then
				result:Add(aspect)
			end

		end

	end

	return result

end

-- ... ===================================================================
-- =======================================================================

function Deck:GetKernelInstance()

	return Kernel:NewInstance(self.kernel)

end

-- EXPOSE ================================================================
-- =======================================================================

function Deck:GetConsole (console)
	
	return self.console

end

-- =======================================================================

function Deck:GetKernel (kernel)
	
	return self.kernel

end

-- =======================================================================

function Deck:GetModules()

	return self.modules

end

-- =======================================================================

function Deck:GetChips()

	return self.chips

end

-- =======================================================================

function Deck:GetFuses()
	
	return self.fuses

end

-- =======================================================================

function Deck:GetVision()
	
	return self.vision

end

-- =======================================================================

function Deck:GetMaxTokens()
	
	return self.maxTokens

end

-- =======================================================================

function Deck:GetStartTokens()
	
	return self.startTokens

end

-- =======================================================================

function Deck:GetTokensPerAccess (class)
	
	return self.tokensPerAccess[class] or 0

end

-- =======================================================================

function Deck:GetTraceSpeedMod()

	return self.traceSpeedMod

end

-- =======================================================================

function Deck:GetTraceSpeedModPerAlert()

	return self.traceSpeedModPerAlert

end

-- =======================================================================

function Deck:GetDetectedSubroutineTraits()

	return self.detections

end

-- =======================================================================

function Deck:GetDownloadSpeedMod()

	return self.downloadSpeedMod

end

-- =======================================================================

function Deck:CanDownloadUnderTrace()
	
	return self.downloadUnderTrace

end

