
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Event = Entity.new()
Event.__index = Event

-- =======================================================================

function Event:__class_setup__ (params)

	self.impulse = params.impulse
	self.format = params.format or EventFormat.Silent
	self.chance = params.chance
	self.useLuckChance = params.useLuckChance
	self.maxExecutionCount = params.maxExecutionCount
	self.cooldownLen = params.cooldown

	-- Choices
	self.choices = Collection.new()
	for c in Collection.new(params.choices or {}):Each() do

		local choice = EventChoice.new(self, c)
		self.choices:Add(choice)

	end

	-- Translates event.choice['accept'] -> event:GetChoiceWithID('accept')
	self.choice = setmetatable({ __event__ = self }, { __index = function(t, ID)

		local choice = t.__event__:GetChoiceWithID(ID)
		return assert(choice, strFormat("The event doesn't contain a choice with ID '%s'.", ID))

	end })

end

-- =======================================================================

function Event:new (values, character)

	local new = {}
	setmetatable(new, self)
	
	new.character = character
	new.values = Values.new(values)
	new.executionCount = 0

	new:Init()
	new:Subscribe()

	return new

end

-- CONSTRUCTION ==========================================================
-- =======================================================================

-- [Event classes can override this]
function Event:Init()
	
end

-- SUBSCRIPTION ==========================================================
-- =======================================================================

function Event:Subscribe()
	
	if self.impulse == nil then
		return
	end

	eventStarter:AddEvent(self)

end

-- =======================================================================

function Event:Unsubscribe()
	
	eventStarter:RemoveEvent(self)

end

-- PREREQUISITES =========================================================
-- =======================================================================

-- [Event classes can override this]
function Event:MeetsPrerequisites()
	
	return true

end

-- CONTEXT ===============================================================
-- =======================================================================

-- [Event classes can override this]
function Event:AcceptsContext (context)
	
	return true

end

-- CHANCE ================================================================
-- =======================================================================

-- [Event classes can override this]
function Event:GetChance()

	return self.chance

end

-- =======================================================================

function Event:SetChance (chance)

	self.chance = chance

end

-- =======================================================================

function Event:UsesLuckChance()
	
	return self.useLuckChance ~= nil

end

-- FORMAT ================================================================
-- =======================================================================

function Event:IsExposed()

	return self.format >= EventFormat.Show

end

-- =======================================================================

function Event:IsResponsive()

	return self.format >= EventFormat.Selection

end

-- EXECUTION =============================================================
-- =======================================================================

function Event:Execute (context)
	
	self:Invoke(context)
	self:IncrementExecutionCount()

	self.cooldown = self.cooldownLen

end

-- =======================================================================

function Event:Invoke (context)

	self:ClearStash()
	self:Run(context)

end

-- =======================================================================

-- [Event classes can override this]
function Event:Run (context)
	
end

-- EXECUTION COUNT =======================================================
-- =======================================================================

function Event:HasMaxExecutionCount()
	
	return self.maxExecutionCount ~= nil

end

-- =======================================================================

function Event:GetMaxExecutionCount()
	
	return self.maxExecutionCount

end

-- =======================================================================

function Event:IncrementExecutionCount()
	
	self.executionCount = self.executionCount + 1

	if self:HasMaxExecutionCount() then
		if self:GetMaxExecutionCount() >= self.executionCount then

			self:Release()

		end
	end

end

-- EXPOSURE ==============================================================
-- =======================================================================

function Event:BuildExposure()
	
	local exposure = EventExposure.new()
	
	-- Title
	exposure:SetTitle(self:GetTitle())

	-- Narrative
	local narrative = self:GetNarrative()
	
	local values = self:ConstructTextValues()
	local evaluatedNarrative = narrative:Evaluate(values)

	exposure:SetNarrative(evaluatedNarrative)

	-- Choices
	for choice in self.choices:Each() do

		local cost = choice:GetCost(self)
		local option = EventOption.new(choice:GetID(), choice:GetCaption(), cost)
		exposure:AddOption(option)

	end

	return exposure

end

-- TITLE =================================================================
-- =======================================================================

function Event:GetTitle()
	
	self.title = self.title or eventTexts:GetEventTitle(self.ID)
	return self.title

end

-- NARRATIVE =============================================================
-- =======================================================================

function Event:GetNarrative()

	self.narrative = self.narrative or eventTexts:GetEventNarrative(self.ID)
	return self.narrative

end

-- =======================================================================

function Event:ConstructTextValues()
	
	local values = self:BuildTextValues()

	if self:IsPersonal() then

		local name = self.character:GetName()
		
		values['__char_name__'] = name:GetNominative()
		values['__char_pronoun__'] = name:GetPronoun()
		values['__char_pronoun_tonique__'] = name:GetToniquePronoun()
		values['__char_pronoun_possessive__'] = name:GetPossessivePronoun()

	end

	return values

end

-- =======================================================================

-- [Event classes can override this]
function Event:BuildTextValues()
	
	return Values.new()

end

-- STASH =================================================================
-- =======================================================================

function Event:ClearStash()

	self.stash = Values.new()

end

-- =======================================================================

function Event:GetStash()
	
	return self.stash

end

-- CHOICES ===============================================================
-- =======================================================================

function Event:ExecuteChoice (choiceID)
	
	local choice = self:GetChoiceWithID(choiceID)
	choice:Execute(self)

end

-- =======================================================================

function Event:GetResponse (choiceID)

	local choice = self:GetChoiceWithID(choiceID)
	local values = choice:GetResponseValues(self)

	return choice:GetResponse():Evaluate(values)

end

-- =======================================================================

function Event:GetChoiceWithID (ID)
	
	return self.choices:First(function (choice) return choice:GetID() == ID end)

end

-- CHARACTER =============================================================
-- =======================================================================

function Event:IsPersonal()

	return self.character ~= nil

end

-- =======================================================================

function Event:GetCharacter()
	
	return self.character

end

-- COOLDOWN ==============================================================
-- =======================================================================

function Event:IsOnCooldown()

	return self.cooldown ~= nil

end

-- GAME TIME =============================================================
-- =======================================================================

function Event:ReceiveTick()

	if self.cooldown then

		self.cooldown = self.cooldown - 1
		if self.cooldown <= 0 then

			self.cooldown = nil

		end

	end

end

-- RELEASE ===============================================================
-- =======================================================================

function Event:Release()

	if self:IsPersonal() then
		self.character:RemoveEvent(self)
	end

	self:Unsubscribe()

end

-- EXPOSE ================================================================
-- =======================================================================

function Event:GetImpulse()
	
	return self.impulse

end

-- =======================================================================

function Event:GetChoices()
	
	return self.choices

end

-- =======================================================================

function Event:GetValues()
	
	return self.values

end

