
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Network = {}
Network.__index = Network

-- =======================================================================

function Network.new (plan)

	local new = {}
	setmetatable(new, Network)
	
	new.nodes = Collection.new()
	new.nodeFactory = NodeFactory.new(new)
	new.connections = Collection.new()
	new.traceStarter = TraceStarter.new(new)
	new.trackers = Collection.new()

	new:CreateFromPlan(plan)
	new:BuildConnectedSubstantialNodes()
	
	return new

end

-- =======================================================================

function Network:CreateFromPlan (plan)
	
	-- Name
	self.name = plan:GetName()

	-- Nodes
	for nodePlan in plan:GetNodePlans():Each() do

		local node = self.nodeFactory:NewNode(nodePlan)
		self.nodes:Add(node)

	end

	-- Connections
	for connectionPlan in plan:GetConnectionPlans():Each() do

		local nodeA = connectionPlan:GetNodeA()
		local nodeB = connectionPlan:GetNodeB()
		
		local nodeA = assert(self:GetNodeWithID(nodeA), strFormat("Node with ID %d is not present in the network.", nodeA))
		local nodeB = assert(self:GetNodeWithID(nodeB), strFormat("Node with ID %d is not present in the network.", nodeB))

		local connection = Connection.new(nodeA, nodeB)
		self.connections:Add(connection)

		nodeA:AddConnectedNode(nodeB)
		nodeB:AddConnectedNode(nodeA)

	end

	-- Trace
	self.traceStarter:LoadFromPlan(plan:GetTracePlan())

end

-- =======================================================================

function Network:LoadFromFile (sourceFilePath)

	local xmlRoot = XmlDocument.newFromFile(sourceFilePath):GetRoot()
	
	-- Name
	self.name = xmlRoot:GetAttribute('name') or '(Unknown)'

	-- Nodes
	local nodesElement = xmlRoot:GetElement('nodes')
	for _, nodeElement in nodesElement:EachElement() do
	
		local node = self.nodeFactory:NewNode(nodeElement)
		self.nodes:Add(node)

	end
	
	-- Connections
	local connectionsElement = xmlRoot:GetElement('connections')
	for _, connectionElement in connectionsElement:EachElement() do
	
		local nodeA = assert(self:GetNodeWithID(connectionElement:GetAttribute('nodeA')), "Connection declared without the 'nodeA' attribute.")
		local nodeB = assert(self:GetNodeWithID(connectionElement:GetAttribute('nodeB')), "Connection declared without the 'nodeB' attribute.")
		
		local connection = Connection.new(nodeA, nodeB)
		self.connections:Add(connection)

		nodeA:AddConnectedNode(nodeB)
		nodeB:AddConnectedNode(nodeA)
	
	end

	-- Trace
	local traceElement = xmlRoot:GetElement('trace')
	self.traceStarter:LoadFromXML(traceElement)

end

-- =======================================================================

function Network:Init()
	
	self:InitVisionBlocks()

	if ticker then
		ticker:ReceivePulse(45)
	end

end

-- CONNECTED NODES =======================================================
-- =======================================================================

function Network:BuildConnectedSubstantialNodes()
	
	local isSubstantial = function (node) return node:IsSubstantial() end
	for node in self.nodes:Select(isSubstantial):Each() do

		local stack = Stack.new()
		local visited = Map.new()

		stack:Push(node)
		visited[node] = true		
		
		node.connectedSubstantialNodes = Collection.new()

		while #stack > 0 do

			local n = stack:Pop()
			for cn in n:GetConnectedNodes():Each() do

				if not visited:ContainsKey(cn) then

					visited[cn] = true
					if cn:IsSubstantial() then
						node.connectedSubstantialNodes:Add(cn)
					else
						stack:Push(cn)
					end

				end

			end

		end

	end

end

-- ACCESS ================================================================
-- =======================================================================

function Network:ExtendAccess (node)

	local stack = Stack.new()

	stack:Push(node)
	while #stack > 0 do

		local n = stack:Pop()
		n:GainAccess()

		if n:AllowsAccessPass() then

			local isNotAccessed = function (node) return not node:IsAccessed() end
			for cn in n:GetConnectedNodes():Select(isNotAccessed):Each() do
				stack:Push(cn)
			end

		end

		local tokens = networkScreen:GetRun():GetTokens()
		tokens:Add(deck:GetTokensPerAccess(n:GetClass()))

	end

end

-- VISION ================================================================
-- =======================================================================

function Network:InitVisionBlocks()

	local isObstacle = function (node) return node:IsObstacle() end
	for node in self.nodes:Select(isObstacle):Each() do

		if node:StartsAsVisionBlock() then
			node:BlockVision()
		end

	end

end

-- =======================================================================

function Network:ExtendVision (node, vision)
	
	local stack = Stack.new()
	local visited = Map.new()
	
	vision = vision or deck:GetVision()

	local maxDepth = 0
	for visibility, depth in pairs(vision) do
		maxDepth = math.max(maxDepth, depth)
	end

	stack:Push({ node = node, depth = 0 })
	while #stack > 0 do

		local item = stack:Pop()

		local n = item.node
		local d = item.depth

		visited[n] = d

		-- For each visibility level:
		for visibility, depth in pairs(vision) do

			if depth >= d then
				n:GainVisibility(visibility)
			end

		end

		local newDepth = d
		if n:IsSubstantial() then
			newDepth = d + 1
		end

		if newDepth <= maxDepth and ((not n:IsObstacle()) or (not n:BlocksVision())) then

			for cn in n:GetConnectedNodes():Each() do

				if (not visited:ContainsKey(cn)) or visited[cn] > newDepth then 
	
					stack:Push({ node = cn, depth = newDepth })

				end

			end

		end

	end

end

-- TRACE =================================================================
-- =======================================================================

function Network:StartTrace (node)

	if self.trace then

		self.traceStarter:ExtendTrace(self.trace, node)

	else

		self.trace = self.traceStarter:NewTrace()
		self.trace:NewPoint(node)
		
	end

	networkScreen:GetRun():ReceiveModuleImpulse(ModuleImpulse.Trace)
	networkScreen:ReceiveTutorialImpulse(TutorialImpulse.Trace)

end

-- =======================================================================

function Network:StopTrace()
	
	if not self.trace then
		return
	end

	self.trace:Terminate()
	self.trace = nil

end

-- =======================================================================

function Network:RunsTrace()
	
	return self.trace ~= nil

end

-- =======================================================================

function Network:GetTrace()
	
	return self.trace

end

-- DISCONNECT ============================================================
-- =======================================================================

function Network:Disconnect()

	for node in self.nodes:Each() do
		node:Disconnect()
	end

	self:StopTrace()

end

-- TRACKER ===============================================================
-- =======================================================================

function Network:StartTracker()

	local evasion
	if random:Chance(0.5) then

		evasion = Evasion:NewInstance('tokens', { tokens = random:IntValue(2, 4) * 2 })

	else
	
		evasion = Evasion:NewInstance('breach', { count = random:IntValue(2, 5) })

	end

	local tracker = Tracker.new(evasion, random:IntValue(4, 6) * random:IntValue(4, 6), self)
	self.trackers:Add(tracker)

end

-- =======================================================================

function Network:StopTracker (tracker)

	self.trackers:RemoveValue(tracker)

end

-- =======================================================================

function Network:ReceiveEvasionImpulse (impulse)

	for tracker in self.trackers:Each() do
		tracker:ReceiveEvasionImpulse(impulse)
	end

end

-- =======================================================================

function Network:GetTrackers()

	return self.trackers

end

-- TIME ==================================================================
-- =======================================================================

function Network:ReceiveTimePulse (pulse)
	
	-- Nodes
	for node in self.nodes:Each() do
		node:ReceiveTimePulse(pulse)
	end

	-- Trace
	if self.trace then

		self.trace:ReceiveTimePulse(pulse)

		-- The Trace could have been stopped now -- check one more time
		if self.trace and self.trace:GetCountdown():IsComplete() then
			self:StopTrace()
		end

	end

	-- Trackers
	for tracker in self.trackers:Each() do

		tracker:ReceiveTimePulse(pulse)

	end
	
end

-- EXPOSE ================================================================
-- =======================================================================

function Network:GetName()
	
	return self.name

end

-- =======================================================================

function Network:GetNodes (class)
	
	if class then
		return self.nodes:Select(function (node) return node:GetClass() == class end)
	end
	
	return self.nodes

end

-- =======================================================================

function Network:GetNodeWithID (ID)

	return self.nodes:First(function (node) return node:GetID() == ID end)

end

-- =======================================================================

function Network:GetEntrance()

	return self.nodes:First(function (node) return node:GetClass() == NodeClass.Entrance end)

end

-- =======================================================================

function Network:GetConnections()

	return self.connections
end

