
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Node = {}
Node.__index = Node

-- =======================================================================

function Node.new (plan)

	local new = {}
	setmetatable(new, Node)
	
	new.visibility = NodeVisibility.None
	new.connectedNodes = Collection.new()
	new.marks = Collection.new()
	
	new:LoadFromPlan(plan)
	
	return new

end

-- CONSTRUCTION ==========================================================
-- =======================================================================

function Node:LoadFromPlan (plan)

	self.ID = plan:GetID()
	
	self.posX = plan:GetPosX()
	self.posY = plan:GetPosY()

end

-- ACCESS ================================================================
-- =======================================================================

function Node:GainAccess()
	
	if self.access then
		return
	end

	self.access = true
	self:ReceiveAccess()

	self:GainVisibility(NodeVisibility.Full)
	self.network:ExtendVision(self)

	self:ReceiveMarkImpulse(MarkImpulse.Access)

	if ticker then
		ticker:ReceivePulse(3)
	end

end

-- =======================================================================

function Node:AllowsAccessPass()
	
	if self:IsObstacle() then
		return not self:BlocksAccess()
	end

	return self.class.passesAccess

end

-- =======================================================================

-- [Node classes can override this]
function Node:ReceiveAccess()

end

-- =======================================================================

function Node:IsAccessed()
	
	return self.access

end

-- VISION ================================================================
-- =======================================================================

function Node:GainVisibility (visibility)

	if visibility <= self.visibility then
		return
	end

	self:ReceiveVisibility(visibility)
	self.visibility = visibility

end

-- =======================================================================

-- [Node classes can override this]
function Node:ReceiveVisibility (visibility)

end

-- =======================================================================

function Node:GetVisibility()
	
	return self.visibility

end

-- TRACE =================================================================
-- =======================================================================

-- [Node classes can override this]
function Node:StartTracePoint (point)
	
end

-- =======================================================================

-- [Node classes can override this]
function Node:ClearTracePoint()
	
end

-- DISCONNECT ============================================================
-- =======================================================================

-- [Node classes can override this]
function Node:Disconnect()

end

-- TIME ==================================================================
-- =======================================================================

-- [Node classes can override this]
function Node:ReceiveTimePulse (pulse)
	
end

-- CONNECTED NODES =======================================================
-- =======================================================================

function Node:GetConnectedNodes (class)

	if class then
		return self.connectedNodes:Select(function (node) return node:GetClass() == class end)
	end
	
	return self.connectedNodes

end

-- =======================================================================

function Node:AddConnectedNode (node)

	self.connectedNodes:Add(node)

end

-- CONNECTED NODES: SUBSTANTIAL ==========================================
-- =======================================================================

function Node:GetConnectedSubstantialNodes (class)

	if class then
		return self.connectedSubstantialNodes:Select(function (node) return node:GetClass() == class end)
	end

	return self.connectedSubstantialNodes

end

-- =======================================================================

function Node:AddConnectedSubstantialNode (node)
	
	self.connectedSubstantialNodes:Add(node)

end

-- NODES WITHIN RADIUS ===================================================
-- =======================================================================

function Node:GetNodesWithinRadius (radius, class)
	
	local result = Map.new()
	local visited = Map.new()
	local stack = Stack.new()

	visited[self] = radius
	stack:Push({ node = self, radius = radius })

	while #stack > 0 do

		local n = stack:Pop()
		local node = n.node

		if (node:GetClass() == class) or (class == nil) then
			result[node] = true
		end

		if n.radius > 0 then

			for cn in node:GetConnectedSubstantialNodes():Each() do

				local v = visited[cn]
				if v == nil or v < n.radius - 1 then

					stack:Push({ node = cn, radius = n.radius - 1 })
					visited[cn] = n.radius - 1

				end

			end

		end

	end

	result = result:ListKeys()
	return result

end

-- MARKS =================================================================
-- =======================================================================

function Node:AddMark (mark)
	
	self.marks:Add(mark)
	mark.host = self

end

-- =======================================================================

function Node:RemoveMark (mark)
	
	self.marks:RemoveValue(mark)

end

-- =======================================================================

function Node:ReceiveMarkImpulse (impulse, context)

	for mark in self.marks:Each() do
		mark:React(impulse, context)
	end

end

-- HINT ==================================================================
-- =======================================================================

-- [Node classes can override this]
function Node:GetHint_Accessed()

	local class = self.class
	return strFormat("[%s]", class.name)

end

-- =======================================================================

-- [Node classes can override this]
function Node:GetHint_Detected()

	local class = self.class
	return strFormat("[%s detected]", class.name)

end

-- EXPOSE ================================================================
-- =======================================================================

function Node:GetID()
	
	return self.ID

end

-- =======================================================================

function Node:GetNetwork()
	
	return self.network

end

-- =======================================================================

function Node:IsObstacle()

	-- Set in the Obstacle constructor
	return self.obstacle

end

-- =======================================================================

function Node:IsSubstantial()
	
	return self.class.substantial

end

-- =======================================================================

function Node:GetNetwork()
	
	return self.network

end

-- =======================================================================

function Node:GetClass()

	return self.class

end

-- =======================================================================

function Node:GetPosX()
	
	return self.posX

end

-- =======================================================================

function Node:GetPosY()
	
	return self.posY

end

