
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Tutorial = Entity.new()
Tutorial.__index = Tutorial

-- =======================================================================

function Tutorial:__class_setup__ (params)

	self.name = params.name
	self.rootDir = 'data/tutorials/' .. params.rootDir

	self.reactions = Map.new()

end

-- =======================================================================

function Tutorial:new()

	local new = {}
	setmetatable(new, self)

	new.hints = Map.new()
	new.modules = Collection.new()

	return new

end

-- LOAD ==================================================================
-- =======================================================================

function Tutorial:Load()

	local xmlRoot = XmlDocument.newFromFile(self.rootDir .. '/tutorial.xml'):GetRoot()

	local deckElement = assert(xmlRoot:GetElement('deck'), strFormat("Tutorial '%s' doesn't contain a <deck> element.", self.ID))
	self:LoadDeck(deckElement)

	local hintsElement = assert(xmlRoot:GetElement('hints'), strFormat("Tutorial '%s' doesn't contain a <hints> element.", self.ID))
	self:LoadHints(hintsElement)

	self.loaded = true

end

-- =======================================================================

function Tutorial:LoadDeck (xmlRoot)
	
	self.console = xmlRoot:GetRequiredAttribute('console')
	self.kernel = xmlRoot:GetRequiredAttribute('kernel')

	local modulesElement = assert(xmlRoot:GetElement('modules'), strFormat("Tutorial '%s' doesn't contain a <modules> element within its <deck> element.", self.ID))
	for _, moduleElement in modulesElement:EachElement('module') do

		local module = moduleElement:GetRequiredAttribute('ID')
		self.modules:Add(module)

	end

end

-- =======================================================================

function Tutorial:LoadHints (xmlRoot)
	
	self.startHint = xmlRoot:GetRequiredAttribute('start')

	for _, hintElement in xmlRoot:EachElement('hint') do

		local ID = hintElement:GetRequiredAttribute('ID')
		local hint = TutorialHint.new(hintElement)

		assert(not self.hints:ContainsKey(ID), strFormat("Duplicate hint ID '%s' declared in tutorial '%s'.", ID, self.ID))
		self.hints[ID] = hint

	end

end

-- =======================================================================

function Tutorial:IsLoaded()
	
	return self.loaded ~= nil

end

-- START =================================================================
-- =======================================================================

function Tutorial:Start()

	self:Init()
	self:ShowHint(self.startHint)

	self.finished = nil

end

-- =======================================================================

-- [Tutorial classes can override this]
function Tutorial:Init()

end

-- REACTIONS =============================================================
-- =======================================================================

function Tutorial:React (impulse, context)

	local reaction = self.reactions[impulse]
	if reaction then

		reaction(self, context)

	end

end

-- HINTS =================================================================
-- =======================================================================

function Tutorial:ShowHint (ID)
	
	local hint = assert(self.hints[ID], strFormat("Tutorial doesn't contain a hint with an ID '%s'.", ID))
	self.hint = hint

	self:React(TutorialImpulse.HintShow, { ID = ID })

end

-- =======================================================================

function Tutorial:GetShownHint() 
	
	return self.hint

end

-- ALTERNATIVE HINT ======================================================
-- =======================================================================

function Tutorial:ShowAlternativeHint (ID)

	local hint = assert(self.hints[ID], strFormat("Tutorial doesn't contain a hint with an ID '%s'.", ID))
	self.alternativeHint = hint

end

-- =======================================================================

function Tutorial:ClearAlternativeHint()
 	
	self.alternativeHint = nil

end 

-- =======================================================================

function Tutorial:ShowsAlternativeHint()
	
	return self.alternativeHint ~= nil

end

-- =======================================================================

function Tutorial:GetShownAlternativeHint()
	
	return self.alternativeHint

end

-- FINISH ================================================================
-- =======================================================================

function Tutorial:Finish()
	
	self.finished = true
	self:Complete()

end

-- =======================================================================

function Tutorial:IsFinished()
	
	return self.finished ~= nil

end

-- COMPLETION ============================================================
-- =======================================================================

function Tutorial:Complete()
	
	self:MarkAsComplete()

	local ps = PersistenceSaver.new()
	ps:Save(consts.persistenceFilePath)

end

-- =======================================================================

function Tutorial:MarkAsComplete()

	self.complete = true

end

-- =======================================================================

function Tutorial:IsComplete()

	return self.complete ~= nil

end

-- UTILITY ===============================================================
-- =======================================================================

function Tutorial:AddModule (ID)
	
	local run = networkScreen:GetRun()
	
	run:CreateModuleAtRuntime(run:GetModules():GetNonBlankCount() + 1, ID)
	networkScreen:RecreateModuleRenders()

end

-- EXPOSE ================================================================
-- =======================================================================

function Tutorial:GetName()
	
	return self.name

end

-- =======================================================================

function Tutorial:GetNetworkSourceFile()
	
	return self.rootDir .. '/network.xml'

end

-- =======================================================================

function Tutorial:GetConsole()
	
	return self.console

end

-- =======================================================================

function Tutorial:GetKernel()
	
	return self.kernel

end

-- =======================================================================

function Tutorial:GetModules()
	
	return self.modules

end
