
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

EventPanel = {}
EventPanel.__index = EventPanel

-- =======================================================================

function EventPanel.new ()

	local new = BaseScreen.new()
	setmetatable(new, EventPanel)

	new:Init()	
	new:Hook()
	new:DisableScreenUI()

	return new

end

-- ... ===================================================================
-- =======================================================================

function EventPanel:Init()
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	self.sizeX = 1140
	self.sizeY = 680
	self.posX = (windowSizeX - self.sizeX) / 2
	self.posY = (windowSizeY - self.sizeY) / 2

end

-- HOOK ==================================================================
-- =======================================================================

function EventPanel:Hook()

	screen:GetCurrent():GetMouseEvents().onMouseButtonPress:Subscribe(self)

end

-- =======================================================================

function EventPanel:Unhook()

	screen:GetCurrent():GetMouseEvents().onMouseButtonPress:Unsubscribe(self)

end

-- SCREEN UI =============================================================
-- =======================================================================

function EventPanel:DisableScreenUI()

	self.disabledScreenControls	= Collection.new()

	for _, control in pairs(loveFrames.base.children) do
		if control:GetEnabled() then

			self.disabledScreenControls:Add(control)
			control:SetEnabled(false)

		end
	end

end

-- =======================================================================

function EventPanel:EnableScreenUI()
	
	for control in self.disabledScreenControls:Each() do
		control:SetEnabled(true)
	end

end

-- ... ===================================================================
-- =======================================================================

function EventPanel:Close()

	screen:GetCurrent():CloseEventPanel()

	self:RemoveUI()
	self:EnableScreenUI()
	self:Unhook()

end

-- ... ===================================================================
-- =======================================================================

function EventPanel:ShowNextEvent()

	local event = eventQueue:GetNextExposedEvent()
	if not event then

		self:Close()
		return

	end

	self.event = event
	self.exposure = event:BuildExposure()

	self.title = self.exposure:GetTitle()

	local narrative = self.exposure:GetNarrative()
	self:ShowNarrative(narrative)

	self:RemoveUI()
	self:CreateUI()

end

-- =======================================================================

function EventPanel:ShowResponse (response)

	self:ShowNarrative(response)
	
	self:RemoveUI()	
	self.options:Clear()

end

-- =======================================================================

function EventPanel:ShowNarrative (narrative)

	self.clauses = narrative:GetClauses()
	self.rewards = narrative:GetRewards()

	for reward in self.rewards:Each() do
		reward:Give()
	end

end

-- CREATE UI =============================================================
-- =======================================================================

function EventPanel:CreateUI()

	local buttonSizeX = 280
	local buttonSizeY = 24

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local spaceY = 32
	local posY = self.posY+self.sizeY - 128 - 64

	self.options = Collection.new()
	if self.exposure:HasOptions() then

		for n, option in self.exposure:GetOptions():EachIndexed() do

			posY = posY + 32

			local button = loveFrames.Create('button')
			button:SetSize(buttonSizeX, buttonSizeY)
			button:SetPos(windowSizeX/2, posY, true)
			button:SetFont('event-choice-button')
			button:SetText(option:GetCaption())
			
			local cost = option:GetCost()
			self.options:Add({ button = button, cost = cost, posY = posY })

			if option:HasCost() then

				button:SetEnabled(credits:CanSpend(option:GetCost()))
				posY = posY + 16

			end

			button.OnClick = function()

				self.event:ExecuteChoice(option:GetID())

				local response = self.event:GetResponse(option:GetID())
				self:ShowResponse(response)

				if option:HasCost() then
					credits:Spend(cost)
				end

			end

		end

	end

end

-- REMOVE UI =============================================================
-- =======================================================================

function EventPanel:RemoveUI()

	if self.options then

		for option in self.options:Each() do
			option.button:Remove()
		end

	end

end	

-- DRAW ==================================================================
-- =======================================================================

function EventPanel:Draw()

	self:DrawBack()
	self:DrawTitle()
	
	local posY 
	posY = self.posY + self.sizeY - 192 - 24
	posY = self:DrawRewards(posY)
	posY = self:DrawClauses(posY)
	self:DrawCharacter(posY)
	
	self:DrawOptions()
	self:DrawCloseButton()
	self:DrawCredits()

end

-- =======================================================================

function EventPanel:DrawBack()
	
	local padd = 16
	love.graphics.setColor(0, 0, 32, 128)
	love.graphics.rectangle('fill', self.posX-padd, self.posY-padd, self.sizeX+2*padd, self.sizeY+2*padd)

	love.graphics.setColor(0, 0, 32, 192)
	love.graphics.rectangle('fill', self.posX, self.posY, self.sizeX, self.sizeY)

	love.graphics.setColor(255, 255, 255, 16)
	local back = sprites:GetSprite('back_event_01')
	love.graphics.draw(back, self.posX, self.posY)

end

-- =======================================================================

function EventPanel:DrawTitle()
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local textSizeX = 480
	fonts:GetFont('event-title'):SetAsLoveFont()
	skin:GetColor('event:title'):SetAsLoveColor()
	love.graphics.printf(strFormat('( %s )', self.title), windowSizeX/2 - textSizeX/2, self.posY+self.sizeY-64, textSizeX, 'center')

end

-- =======================================================================

function EventPanel:DrawRewards (posY)

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local rectSizeX = 820
	local rectSizeY = 32
	local spaceY = 2

	local font = fonts:GetFont('event-reward')
	local fontHeight = font:GetHeight()
	font:SetAsLoveFont()

	for reward in self.rewards:Each() do

		local text = ''
		local class = reward:GetClass()

		if class == RewardClass.Credits then
			text = strFormat("You receive ¥%d", reward:GetAmount())
		end

		love.graphics.setColor(128, 128, 232, 64)
		love.graphics.rectangle('fill', windowSizeX/2 - rectSizeX/2, posY-rectSizeY, rectSizeX, rectSizeY)

		local textSizeX = 480
		love.graphics.setColor(168, 168, 232, 232)
		love.graphics.printf(text, windowSizeX/2 - textSizeX/2, posY - rectSizeY/2 - fontHeight/2, textSizeX, 'center')

		posY = posY - rectSizeY - spaceY

	end

	if #self.rewards > 0 then
		posY = posY - 32
	end

	return posY

end

-- =======================================================================

function EventPanel:DrawClauses (posY)
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local textSizeX = 820
	local clauseSpaceY = 12

	local font = fonts:GetFont('event-clause')
	local fontHeight = font:GetHeight()
	font:SetAsLoveFont()

	for c in self.clauses:ReverseEach() do

		local kind = c:GetKind()
		if kind == ClauseKind.DirectSpeech then

			skin:GetColor('event:clause-direct'):SetAsLoveColor()

		else

			skin:GetColor('event:clause'):SetAsLoveColor()

		end

		local text = c:GetText()
		local width, lines = font:CalculateSize(text, textSizeX)
		local clauseSizeY = lines * fontHeight

		love.graphics.printf(text, windowSizeX/2 - textSizeX/2, posY-clauseSizeY, textSizeX, 'center')
		posY = posY - clauseSizeY - clauseSpaceY

	end

	return posY

end

-- =======================================================================

function EventPanel:DrawCharacter (posY)
	
	if not self.event:IsPersonal() then
		return
	end

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local posX = windowSizeX/2
	posY = posY - 92 - 24

	local radius = 80
	love.graphics.setLineWidth(1)
	love.graphics.setColor(144, 144, 232, 16)
	love.graphics.circle('fill', posX, posY, radius, 64)
	love.graphics.setColor(144, 144, 232, 64)
	love.graphics.circle('line', posX, posY, radius, 64)

	love.graphics.setColor(144, 144, 232, 232)
	fonts:GetFont('character-name'):SetAsLoveFont()
	love.graphics.printf(self.event:GetCharacter():GetName():GetNominative(), posX-radius, posY-6, 2*radius, 'center')

	return posY

end

-- =======================================================================

function EventPanel:DrawOptions()
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	for option in self.options:Each() do

		if option.cost then

			local sizeX = 48
			love.graphics.setColor(32, 32, 92)
			love.graphics.rectangle('fill', windowSizeX/2-sizeX, option.posY, 2*sizeX, 24+4+2)

			local sizeX = 56
			love.graphics.rectangle('fill', windowSizeX/2-sizeX, option.posY, 2*sizeX, 20+4+1)

			local sizeX = 64
			love.graphics.rectangle('fill', windowSizeX/2-sizeX, option.posY, 2*sizeX, 16+4)

			love.graphics.setColor(192, 192, 232, 232)
			fonts:GetFont('event-option-cost'):SetAsLoveFont()
			love.graphics.printf(strFormat('( ¥%d )', option.cost), windowSizeX/2-100, option.posY+12+2, 200, 'center')

		end

		option.button:draw()

	end

end

-- =======================================================================

function EventPanel:DrawCloseButton()

	if #self.options > 0 then

		self.closeButton = nil
		return

	end

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local posX = windowSizeX/2
	local posY = self.posY + self.sizeY - 128 
	local size = 16

	self.closeButton = { posX = posX, posY = posY, size = size }

	local color = skin:GetColor('event:close-button')
	color:SetAsLoveColor(pick(self:IsCloseButtonHovered(), 232, 168))

	love.graphics.setLineWidth(1)
	love.graphics.line(posX-size, posY-size, posX+size, posY+size)
	love.graphics.line(posX-size, posY+size, posX+size, posY-size)

end

-- =======================================================================

function EventPanel:DrawCredits()

	for option in self.options:Each() do
		if option.cost then

			screen:GetCurrent():DrawCredits()
			return

		end
	end

end

-- MOUSE =================================================================
-- =======================================================================

function EventPanel:OnMouseButtonPress (button, curX, curY)

	if button == MouseButton.Left then
		if self.closeButton and self:IsCloseButtonHovered() then

			self:ShowNextEvent()

		end
	end

end

-- =======================================================================

function EventPanel:IsCloseButtonHovered()

	local curX, curY = mouse:GetCursorPos()
	return math.abs(curX-self.closeButton.posX) < self.closeButton.size and math.abs(curY-self.closeButton.posY) < self.closeButton.size

end

