
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

ModuleRender = {}
ModuleRender.__index = ModuleRender

-- =======================================================================

function ModuleRender.new (module, posX, posY)

	local new = {}
	setmetatable(new, ModuleRender)
	
	new.posX = posX
	new.posY = posY
	new.module = module

	new:CreateUI()

	return new

end

-- =======================================================================

function ModuleRender:CreateUI()
	
	local buttonSizeX = 145
	local buttonSizeY = 32

	self.button = loveFrames.Create('button')
	self.button:SetSize(buttonSizeX, buttonSizeY)
	self.button:SetPos(self.posX, self.posY, true)
	self.button:SetText(self.module:GetName():upper())
	self.button:SetFont('module-name')

	local module = self.module
	self.button.OnClick = function (button)

		if not module:CanExecute() then
			return
		end

		local moduleUse = module:GetUse()
		if moduleUse == ModuleUse.Instant then

			module:Execute(network)

		else

			local networkControl = networkScreen:GetNetworkControl()
			networkControl:SetActiveModule(module)

		end

	end

	local unlockButtonSizeX = 95
	local unlockButtonSizeY = 16

	self.unlockButton = loveFrames.Create('button')
	self.unlockButton:SetSize(unlockButtonSizeX, unlockButtonSizeY)
	self.unlockButton:SetPos(self.posX, self.posY - 16, true)
	self.unlockButton:SetText('UNLOCK')
	self.unlockButton:SetFont('module-unlock-button')

	local module = self.module
	self.unlockButton.OnClick = function (button)

		local tokens = networkScreen:GetRun():GetTokens()
		local unlockCost = module:GetUnlockCost()
		
		tokens:Remove(unlockCost)
		module:Unlock()

	end

end

-- =======================================================================

function ModuleRender:RemoveUI()
	
	self.button:Remove()
	self.unlockButton:Remove()

end

-- DRAW ==================================================================
-- =======================================================================

function ModuleRender:Draw()

	if self.button:GetHover() then
		self:DrawDescription()
	end

	if self.module:RequiresTokens() then

		local posY = self.posY+16
		local tokens = self.module:GetRequiredTokens()
		local color = not self.module:IsLocked()

		self:DrawTokens(tokens, posY, color)

	end

	self.button:draw()

	local networkControl = networkScreen:GetNetworkControl()
	if networkControl:GetActiveModule() == self.module then

		local button = self.button

		love.graphics.setLineWidth(3)
		skin:GetColor('network:selected-module-border'):SetAsLoveColor()

		local border = 1
		local buttonSizeX = button:GetWidth()
		local buttonSizeY = button:GetHeight()

		love.graphics.rectangle('line', button:GetX()-border, button:GetY()-border, buttonSizeX+2*border-1, buttonSizeY+2*border)

	end

	if love.keyboard.isDown('lalt') and self.shortcut then
		self:DrawShortcut()
	end

	self:DrawLock()

end

-- =======================================================================

function ModuleRender:DrawDescription()

	local module = self.module

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local traitLine = ''
	for trait in module:GetTraits():Each() do

		if traitLine:len() > 0 then
			traitLine = traitLine .. ' | '
		end

		traitLine = traitLine .. trait.label

	end

	local xSize = 625
	skin:GetColor('network:hint'):SetAsLoveColor()

	local posY = self.posY + 18 + 16 + 4
	if networkScreen:GetNetwork():RunsTrace() then

		posY = posY - 8 - 2

	end

	fonts:GetFont('module-description'):SetAsLoveFont()
	love.graphics.printf(module:GetRuntimeDescription(), windowSizeX/2 - xSize, posY, 2*xSize, 'center')

	fonts:GetFont('module-traits'):SetAsLoveFont()
	love.graphics.printf(traitLine, windowSizeX/2 - xSize, posY + 18, 2*xSize, 'center')

end

-- =======================================================================

function ModuleRender:DrawTokens (count, posY, color)

	local spaceX = 20
	local startX = self.posX - (count-1)*spaceX/2

	for n = 1, count do

		local posX = startX + (n-1)*spaceX

		love.graphics.setLineWidth(1)
		skin:GetColor(pick(color, 'network:module-token', 'network:module-token-locked')):SetAsLoveColor()
		love.graphics.circle('fill', posX, posY, 9, 6)
		skin:GetColor('network:module-token-outline'):SetAsLoveColor()
		love.graphics.circle('line', posX, posY, 9, 6)

	end

end

-- =======================================================================

function ModuleRender:DrawLock()

	if self.module:IsLocked() then

		self.button:SetEnabled(false)
		self.unlockButton:SetVisible(true)

		local unlockCost = self.module:GetUnlockCost()
		self:DrawTokens(unlockCost, self.posY-24, true)

		local tokens = networkScreen:GetRun():GetTokens()
		self.unlockButton:SetEnabled(tokens:CanSpend(unlockCost))

		self.unlockButton:draw()

	else

		self.button:SetEnabled(true)
		self.unlockButton:SetVisible(false)

	end

end

-- =======================================================================

function ModuleRender:DrawShortcut()

	fonts:GetFont('module-description'):SetAsLoveFont()
	love.graphics.setColor(128, 128, 128, 232)
	love.graphics.printf(strFormat('[%s]', self.shortcut:upper()), self.posX-20, self.posY-32, 40, 'center')

end

-- SHORTCUT ==============================================================
-- =======================================================================

function ModuleRender:SetShortcut (shortcut)

	self.shortcut = shortcut

end

-- =======================================================================

function ModuleRender:GetShortcut()

	return self.shortcut

end

-- =======================================================================

function ModuleRender:ExecuteShortcut()
	
	if love.keyboard.isDown('lctrl') then

		self:ExecuteShortcut_Unlock()

	else

		self:ExecuteShortcut_Run()
		networkScreen:ReceiveTutorialImpulse(TutorialImpulse.ModuleExecutionViaShortcut, {})

	end

end

-- =======================================================================

function ModuleRender:ExecuteShortcut_Run()
	
	local module = self.module
	if not module:CanExecute() then
		return
	end

	local use = module:GetUse()
	if use == ModuleUse.Instant then

		module:Execute()

	end

	local curX, curY = mouse:GetCursorPos()
	if use == ModuleUse.Node then

		local node = networkScreen.networkRender:GetNodeOnScreenCoords(curX, curY)

		if node then
			if module:CanExecuteOnNode(node) then
				module:ExecuteOnNode(node)
			end
		else

			local networkControl = networkScreen:GetNetworkControl()
			networkControl:SetActiveModule(module)

		end

	end

	if use == ModuleUse.Subroutine then

		local subroutine = networkScreen.networkRender:GetSubroutineOnScreenCoords(curX, curY)

		if subroutine then
			if module:CanExecuteOnSubroutine(subroutine) then
				module:ExecuteOnSubroutine(subroutine)
			end
		else

			local networkControl = networkScreen:GetNetworkControl()
			networkControl:SetActiveModule(module)

		end

	end

end

-- =======================================================================

function ModuleRender:ExecuteShortcut_Unlock()
	
	local module = self.module
	if not module:IsLocked() then
		return
	end

	local tokens = networkScreen:GetRun():GetTokens()
	local unlockCost = module:GetUnlockCost()
	
	if tokens:CanSpend(unlockCost) then
	
		tokens:Remove(unlockCost)
		module:Unlock()

	end
	
end

-- ... ===================================================================
-- =======================================================================

function ModuleRender:IsHovered()

	return self.button:GetHover()	

end

