
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

NetworkRender = {}
NetworkRender.__index = NetworkRender

-- =======================================================================

function NetworkRender.new (network)

	local new = {}
	setmetatable(new, NetworkRender)

	new.network = network

	new.nodeRenderFactory = NodeRenderFactory.new()

	new.nodeRenders = {}
	new.connectionRenders = {}

	new:CreateNodeRenders()
	new:CreateConnectionRenders()

	new.trackerRenders = Map.new()

	new.effects = Effects.new()

	return new

end

-- ... ===================================================================
-- =======================================================================

function NetworkRender:CreateNodeRenders()

	for node in self.network:GetNodes():Each() do

		local nodeRender = self.nodeRenderFactory:NewNodeRender(node)
		self.nodeRenders[node] = nodeRender

	end

end

-- =======================================================================

function NetworkRender:CreateConnectionRenders()

	for connection in self.network:GetConnections():Each() do

		local connectionRender = ConnectionRender.new(self, connection)
		table.insert(self.connectionRenders, connectionRender)

	end

end

-- ... ===================================================================
-- =======================================================================

function NetworkRender:Release()
	
	for _, nodeRender in pairs(self.nodeRenders) do
		nodeRender:Release()
	end

end

-- DRAW ==================================================================
-- =======================================================================

function NetworkRender:Draw()
	
	self:DrawBack()

	-- Effects
	self.effects:RemoveTerminated()
	self.effects:Draw()

	-- Connections
	for _, connectionRender in ipairs(self.connectionRenders) do
		connectionRender:Draw()
	end

	-- Nodes
	for _, nodeRender in pairs(self.nodeRenders) do

		local node = nodeRender.node
		local visibility = node:GetVisibility()

		if visibility > NodeVisibility.None then

			if visibility == NodeVisibility.Layout then

				local screenX, screenY = nodeRender:GetScreenCoords()

				love.graphics.setLineWidth(1)
				love.graphics.setColor(168, 168, 168, 168)
				love.graphics.circle('line', screenX, screenY, 2, 4)

			else

				nodeRender:Render()

			end

		end

	end

	-- Gradient
	self:DrawGradient()

	-- Trace
	if self.traceRender then
		self.traceRender:Draw()
	end

	-- Trackers
	self:DrawTrackers()

	-- Name
	self:DrawName()

end

-- =======================================================================

function NetworkRender:DrawBack()

	local lineSpace = 50
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setLineWidth(1)	
	skin:GetColor('network:back-line'):SetAsLoveColor()

	local parallaxX = math.ceil((networkScreen.networkControl:GetCameraControl():GetPosX() / 4) % lineSpace)
	local parallaxY = math.ceil((networkScreen.networkControl:GetCameraControl():GetPosY() / 4) % lineSpace)

	for posY = lineSpace-parallaxY, windowSizeY, lineSpace do
		love.graphics.line(0, posY, windowSizeX, posY)
	end
	for posX = lineSpace-parallaxX, windowSizeX, lineSpace do
		love.graphics.line(posX, 0, posX, windowSizeY)
	end

end

-- =======================================================================

function NetworkRender:DrawGradient()

	local alphaBase = 64
	local alphaStep = 16

	local sizeYBase = 36
	local sizeYStep = 8

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local color = skin:GetColor('network:gradient')

	local steps = 10
	for n = 1, steps do

		local alpha = alphaBase + n*alphaStep
		color:SetAsLoveColor(alpha)

		local sizeY = sizeYBase + (steps-n)*sizeYStep
		love.graphics.rectangle('fill', 0, windowSizeY-sizeY, windowSizeX, sizeY)

	end

end

-- =======================================================================

function NetworkRender:DrawTrackers()

	local sizeY = 64
	local spaceY = 2

	local trackers = self.network:GetTrackers()
	for n, tracker in trackers:ReverseEachIndexed() do

		local trackerRender = self.trackerRenders[tracker] 
		if not trackerRender then

			trackerRender = TrackerRender.new(tracker)
			self.trackerRenders[tracker] = trackerRender

		end
	
		local posY = (n-1) * (sizeY + spaceY) + 2
		trackerRender:Draw(posY, sizeY)

	end

	for tracker, trackerRender in self.trackerRenders:Each() do

		if not trackers:Contains(tracker) then

			self.trackerRenders:RemoveKey(tracker)
			trackerRender:Release()

		end

	end

end

-- =======================================================================

function NetworkRender:DrawName()
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local text = self.network:GetName()

	fonts:GetFont('network-name'):SetAsLoveFont()
	skin:GetColor('network:network-name'):SetAsLoveColor()
	love.graphics.printf(text, windowSizeX-400-32, windowSizeY-128-12-6, 400, 'right')

end

-- ... ===================================================================
-- =======================================================================

function NetworkRender:GetNodeOnScreenCoords (screenX, screenY)
	
	for _, nodeRender in pairs(self.nodeRenders) do

		if nodeRender:GetNode():GetVisibility() > NodeVisibility.None then

			local nodeScreenX, nodeScreenY = nodeRender:GetScreenCoords()
			if math.sqrt(math.pow(screenX-nodeScreenX, 2)+math.pow(screenY-nodeScreenY, 2)) < 20 then
				return nodeRender:GetNode()
			end

		end

	end

end

-- =======================================================================

function NetworkRender:GetSubroutineOnScreenCoords (screenX, screenY)
	
	for _, nodeRender in pairs(self.nodeRenders) do

		if nodeRender:GetNode():GetClass() == NodeClass.Router then

			for _, subroutineRender in ipairs(nodeRender:GetSubroutineRenders()) do

				if subroutineRender:IsOnScreenCoords(screenX, screenY) then
					return subroutineRender:GetSubroutine()
				end

			end

		end

	end

end

-- EFFECTS ===============================================================
-- =======================================================================

function NetworkRender:AddEffect (effect)
	
	self.effects:Add(effect)

end

-- TIME ==================================================================
-- =======================================================================

function NetworkRender:ReceiveTimePulse (pulse)
	
	self.effects:ReceiveTimePulse(pulse)

	for _, nodeRender in pairs(self.nodeRenders) do
		nodeRender:ReceiveTimePulse(pulse)
	end

end

-- MOUSE =================================================================
-- =======================================================================

function NetworkRender:OnMouseButtonPress (button, curX, curY)

end

-- EXPOSE ================================================================
-- =======================================================================

function NetworkRender:GetNodeRender (node)

	return self.nodeRenders[node]

end

