
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

BackstoriesScreen = {}
BackstoriesScreen.__index = BackstoriesScreen

-- =======================================================================

setmetatable(BackstoriesScreen, BaseScreen)

-- =======================================================================

BackstoriesScreen.__static__ = 
{
	explanationID = 'backstories'
}

-- =======================================================================

function BackstoriesScreen.new()

	local new = BaseScreen.new()
	setmetatable(new, BackstoriesScreen)

	new:Hook()

	return new

end

-- ENTER/LEAVE ===========================================================
-- =======================================================================

function BackstoriesScreen:Enter (context)

	self:CreateUI()
	self.selection = Collection.new()
	
	self.karma = 5
	
	local isComplete = function (a) return a:IsComplete() end
	self.karma = self.karma + #achievements:GetAspirations():Select(isComplete)

end

-- =======================================================================

function BackstoriesScreen:Leave()

	self:RemoveUI()

end

-- CREATE UI =============================================================
-- =======================================================================

function BackstoriesScreen:CreateUI()

	self:CreateBackstories()
	self:CreateButtons()

end

-- =======================================================================

function BackstoriesScreen:CreateBackstories()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local lockedSizeY = 72
	local unlockedSizeY = 92
	local spaceY = 12
	local sizeX = 960
	local posY = windowSizeY - 92 - 48 - 32 - 8
	
	self.backstories = Collection.new()
	local unlockedFirstComparator = function (bs1, bs2) return bs1:IsUnlocked() and (not bs2:IsUnlocked()) end
	for n, back in backstories:GetBackstories():Sort(unlockedFirstComparator):EachIndexed() do

		local unlocked = back:IsUnlocked()
		local sizeY = pick(unlocked, unlockedSizeY, lockedSizeY)

		self.backstories:Add({ posY = posY, backstory = back, unlocked = unlocked, sizeY = sizeY, sizeX = sizeX })

		posY = posY - (sizeY + spaceY)

	end

end

-- =======================================================================

function BackstoriesScreen:CreateButtons()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	-- 'Start' button
	self.startButton = loveFrames.Create('button')
	self.startButton:SetSize(720, 48)
	self.startButton:SetPos(windowSizeX/2, windowSizeY-64-32-24, true)
	self.startButton:SetText("[ Start ]")
	self.startButton:SetFont('menu-button')

	self.startButton.OnClick = function()

		local isSelected = function (bs) return self:IsBackstorySelected(bs.backstory) end
		local backstories = self.backstories:Select(isSelected):Map(function (bs) return bs.backstory end)

		local newGameStarter = NewGameStarter.new(backstories)
		newGameStarter:StartNewGame()

		screen:Set(characterScreen)

	end

	-- 'Back' button
	self.backButton = loveFrames.Create('button')
	self.backButton:SetSize(420, 24+4)
	self.backButton:SetPos(windowSizeX/2, windowSizeY-64-8+32-2-24, true)
	self.backButton:SetText("[ Back to Menu ]")
	self.backButton:SetFont('menu-button-small')

	self.backButton.OnClick = function()

		screen:Set(menuScreen)

	end

end

-- REMOVE UI =============================================================
-- =======================================================================

function BackstoriesScreen:RemoveUI()

	self.startButton:Remove()
	self.backButton:Remove()

end	

-- DRAW ==================================================================
-- =======================================================================

function BackstoriesScreen:Draw()

	self:DrawBackTexture(1)
	self:DrawBackGradient(4, 16, 12, 16, 8)
	self:DrawBackstories()
	self:DrawKarma()
	self:DrawStartButton()
	self:DrawExplanationButton()

end

-- =======================================================================

function BackstoriesScreen:DrawBackstories()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local curX, curY = mouse:GetCursorPos()
	local hoverBack = self:GetBackstoryOnPos(curX, curY)
	
	for n, back in self.backstories:EachIndexed() do

		local posY = back.posY
		local sizeX = back.sizeX
		local sizeY = back.sizeY

		local backstory = back.backstory
		local unlocked = back.unlocked
		local selected = self:IsBackstorySelected(backstory)
		local hover = (hoverBack == back) and unlocked
		
		-- Back
		if unlocked then
			love.graphics.setColor(64, 128, 232, pick(selected, 32, 8) * pick(hover, 1.25, 1))
		else
			love.graphics.setColor(128, 128, 168, 32)
		end
		
		love.graphics.rectangle('fill', windowSizeX/2-sizeX/2, posY-sizeY, sizeX, sizeY)

		-- Border
		if unlocked then

			love.graphics.setLineWidth(1)
			love.graphics.setColor(64, 64, pick(selected, 192, 128), pick(hover, 192, 128))
			love.graphics.rectangle('line', windowSizeX/2-sizeX/2, posY-sizeY, sizeX, sizeY)

		end

		-- Text color
		local color = Color.newFromRGB(64, 192, 255)

		-- Name
		fonts:GetFont('backstory-name'):SetAsLoveFont()
		color:SetAsLoveColor(pick(selected, 232, pick(hover, 162, 128)))
		love.graphics.printf(backstory:GetName(), windowSizeX/2-sizeX/2, posY-72-8 + pick(unlocked, pick(selected or hover, 0, 12), 24-2), sizeX, 'center')
		
		if unlocked then

			-- Description
			fonts:GetFont('backstory-description'):SetAsLoveFont()
			color:SetAsLoveColor(pick(selected, 232, pick(hover, 162, 128)))
			love.graphics.printf(backstory:GetDescription(), windowSizeX/2-sizeX/2, posY-64+4 + pick(selected or hover, 0, 12), sizeX, 'center')
			
			-- Comment
			if selected or hover then

				fonts:GetFont('backstory-comment'):SetAsLoveFont()
				color:SetAsLoveColor(pick(selected, 192, pick(hover, 128, 96)))
				love.graphics.printf(backstory:GetComment(), windowSizeX/2-sizeX/2, posY-32, sizeX, 'center')

			end

			-- Karma
			local spaceY = 24
			local karma = backstory:GetKarma()
			local startY = posY - sizeY/2 - (math.abs(karma)-1) * spaceY / 2
			local positive = karma < 0
			local posX = windowSizeX/2 - sizeX/2 + 32

			for n = 1, math.abs(karma) do
			
				local posY = startY + (n-1)*spaceY
				self:DrawKarmaPoint(posX, posY, positive)
			
			end

		else

			local achievement = achievements:GetAchievement(backstory:GetAchievement())
			local text = strFormat('[%s to unlock]', achievement:GetDescription())

			color:SetAsLoveColor(128)
			fonts:GetFont('backstory-unlock-label'):SetAsLoveFont()
			love.graphics.printf(text, windowSizeX/2-sizeX/2, posY-32-2, sizeX, 'center')

		end
		
	end

end

-- =======================================================================

function BackstoriesScreen:DrawKarma()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local posX = 128
	local posY = windowSizeY - 128 - 32
	local radius = 24

	love.graphics.setColor(64, 192, 255, 128)
	love.graphics.circle('fill', posX, posY, radius, 4)

	love.graphics.setLineWidth(1)
	love.graphics.setColor(192, 232, 255, 72)
	love.graphics.circle('line', posX, posY, radius, 4)
	love.graphics.circle('line', posX, posY, radius-4, 4)

	love.graphics.setColor(232, 232, 232, 192)
	fonts:GetFont('karma-count'):SetAsLoveFont()
	love.graphics.printf(toStr(self.karma), posX-radius, posY-6, 2*radius, 'center')
	love.graphics.printf("karma", posX-radius, posY+24+4+4, 2*radius, 'center')

end

-- =======================================================================

function BackstoriesScreen:DrawKarmaPoint (screenX, screenY, positive)

	local radius = 10

	love.graphics.setColor(64, 192, 255, 128)
	love.graphics.circle('fill', screenX, screenY, radius, 4)
	
	love.graphics.setLineWidth(1)
	love.graphics.setColor(192, 232, 255, 72)
	love.graphics.circle('line', screenX, screenY, radius-1, 4)

	if positive then
	
		local size = radius - 2

		love.graphics.setLineWidth(1)
		love.graphics.setColor(128, 232, 128, 192)
		love.graphics.line(screenX-size, screenY, screenX+size, screenY)
		love.graphics.line(screenX, screenY-size, screenX, screenY+size)
	
	end
	
end

-- =======================================================================

function BackstoriesScreen:DrawStartButton()

	self.startButton:draw()
	self.backButton:draw()

end

-- SELECTION =============================================================
-- =======================================================================

function BackstoriesScreen:SelectBackstory (back)

	self.selection:Add(back)

end

-- =======================================================================

function BackstoriesScreen:DeselectBackstory (back)

	self.selection:RemoveValue(back)

end

-- =======================================================================

function BackstoriesScreen:IsBackstorySelected (back)

	return self.selection:Contains(back)

end

-- MOUSE =================================================================
-- =======================================================================

function BackstoriesScreen:OnMouseButtonPress (button, curX, curY)

	self:CheckExplanationButtonPress(button, curX, curY)

	if button == MouseButton.Left then

		local back = self:GetBackstoryOnPos(curX, curY)
		if back and back.unlocked then
		
			local backstory = back.backstory
			if self:IsBackstorySelected(backstory) then
			
				self:DeselectBackstory(backstory)
				self.karma = self.karma + backstory:GetKarma()
			
			else
			
				local karma = backstory:GetKarma()
				if self.karma >= karma then
				
					self:SelectBackstory(backstory)
					self.karma = self.karma - karma
					
				end
			
			end
		
		end
		
	end

end

-- =======================================================================

function BackstoriesScreen:GetBackstoryOnPos (curX, curY)

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	for back in self.backstories:Each() do

		local posY = back.posY
		local sizeX = back.sizeX
		local sizeY = back.sizeY

		if (curX > windowSizeX/2-sizeX/2) and (curX < windowSizeX/2+sizeX/2) and (curY < posY) and (curY > posY-sizeY) then
			return back
		end

	end

end

