
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

BaseScreen = {}
BaseScreen.__index = BaseScreen

-- =======================================================================

function BaseScreen.new()

	local new = {}
	setmetatable(new, BaseScreen)

	new.screenButtonRenders = Map.new()
	new.mouseEvents = MouseEvents.new()

	return new

end

-- ... ===================================================================
-- =======================================================================

function BaseScreen:Set()

	mouse:SetEvents(self.mouseEvents)

end

-- HOOK ==================================================================
-- =======================================================================

function BaseScreen:Hook()

	self:GetMouseEvents().onMouseButtonPress:Subscribe(self)

end

-- =======================================================================

function BaseScreen:Unhook()

	self:GetMouseEvents().onMouseButtonPress:Unsubscribe(self)

end

-- ENTER / LEAVE =========================================================
-- =======================================================================

-- [Screen classes can override this]
function BaseScreen:Enter (context)

end

-- =======================================================================

-- [Screen classes can override this]
function BaseScreen:Leave()

end

-- ... ===================================================================
-- =======================================================================

function BaseScreen:ReceiveTimePulse (pulse)

end

-- DRAW ==================================================================
-- =======================================================================

function BaseScreen:DrawBackGradient (steps, borderStepX, borderStepY, alphaBase, alphaStep)
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	for n = 1, steps do

		local borderX = n*borderStepX
		local borderY = n*borderStepY

		love.graphics.setColor(4, 4, 64, alphaBase+n*alphaStep)
		love.graphics.rectangle('fill', borderX, borderY, windowSizeX-2*borderX, windowSizeY-2*borderY)

	end

end

-- =======================================================================

function BaseScreen:DrawBackTexture (index)
	
	love.graphics.setColor(255, 255, 255)
	local back = sprites:GetSprite(strFormat('back_%02d', index))
	love.graphics.draw(back, 0, 0)

end

-- =======================================================================

function BaseScreen:DrawCredits()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setColor(32, 32, 32, 232)
	love.graphics.circle('fill', windowSizeX/2, windowSizeY+80, 120, 6) 

	love.graphics.setLineWidth(3)
	love.graphics.setColor(232, 232, 232, 232)
	love.graphics.circle('line', windowSizeX/2, windowSizeY+80, 120, 6) 

	fonts:GetFont('credits-balance'):SetAsLoveFont()
	love.graphics.printf(strFormat('¥%d', credits:GetBalance()), 0, windowSizeY-18+2, windowSizeX, 'center')

end

-- BOTTOM BUTTONS ========================================================
-- =======================================================================

function BaseScreen:DrawBottomButton (posX, border, render, text)

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local sizeY = 24
	local borderShort = border - 24

	local hover = (render ~= nil) and self:IsBottomButtonHovered(render)

	local color = pick(hover, 48, 32)
	love.graphics.setColor(color, color, 3 * color, 232)
	love.graphics.polygon('fill', posX-border/2, windowSizeY, posX-borderShort/2, windowSizeY-sizeY, posX+borderShort/2, windowSizeY-sizeY, posX+border/2, windowSizeY)

	color = color * 1.2
	love.graphics.setLineWidth(1)
	love.graphics.setColor(color, color, 3 * color, 232)
	love.graphics.polygon('line', posX-border/2, windowSizeY, posX-borderShort/2, windowSizeY-sizeY, posX+borderShort/2, windowSizeY-sizeY, posX+border/2, windowSizeY)

	love.graphics.setColor(128, 192, 232, pick(hover, 192, 128))
	fonts:GetFont('bottom-button'):SetAsLoveFont()
	love.graphics.printf(text, posX-border, windowSizeY-sizeY+4, 2*border, 'center')

	return { sizeY = sizeY, border = (border + borderShort)/2, posX = posX }

end

-- =======================================================================

function BaseScreen:IsBottomButtonHovered (render)

	if not render then
		return
	end
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local curX, curY = mouse:GetCursorPos()
	return math.abs(curX - render.posX) < render.border/2 and curY > windowSizeY - render.sizeY

end

-- SCREEN BUTTONS ========================================================
-- =======================================================================

function BaseScreen:DrawScreenButtons()

	local screens = Collection.new({ connectionScreen, characterScreen, deckScreen })

	local windowSizeY = love.window.getHeight()
	local centerY = windowSizeY - 168
	local spaceY = 72

	local startY = centerY - (#screens-1)*spaceY/2
	for n, screen in screens:EachIndexed() do

		local posY = startY + (n-1)*spaceY
		self:DrawScreenButton(screen, posY)

	end

end

-- =======================================================================

function BaseScreen:DrawScreenButton (screen, posY)

	local width = 24
	local border = 72
	local borderShort = 48

	self.screenButtonRenders[screen] = { width = width, border = (border + borderShort)/2, posY = posY }
	local hover = (self:GetHoveredScreenButton() == screen)

	local color = 32
	if hover then
		color = 48
	end

	if screen == self then
		color = 64
	end

	love.graphics.setColor(color, color, 3 * color, 232)
	love.graphics.polygon('fill', 0, posY-border/2, width, posY-borderShort/2, width, posY+borderShort/2, 0, posY+border/2) 

	local color = 128
	if screen == self then
		color = 192
	end

	love.graphics.setLineWidth(1)
	love.graphics.setColor(color, color, math.min(1.6 * color, 255), 232)
	love.graphics.polygon('line', 0, posY-border/2, width, posY-borderShort/2, width, posY+borderShort/2, 0, posY+border/2) 

	local text = ''
	if screen == connectionScreen then

		self:DrawScreenButtonIcon_Connection(width/2, posY, screen, hover)
		text = "Coordinates"

	end

	if screen == characterScreen then

		self:DrawScreenButtonIcon_Character(width/2, posY, screen, hover)
		text = "Contacts"

	end

	if screen == deckScreen then

		self:DrawScreenButtonIcon_Deck(width/2, posY, screen, hover)
		text = "Deck"

	end

	self:DrawScreenButtonIcon(width/2, posY, screen, hover)

	if hover then

		local windowSizeX = love.window.getWidth()
		local windowSizeY = love.window.getHeight()

		love.graphics.setColor(128, 128, 232, 232)
		fonts:GetFont('screen-switchboard'):SetAsLoveFont()
		love.graphics.print(strFormat('< %s', text):upper(), 12, windowSizeY-24)

	end

end

-- =======================================================================

function BaseScreen:DrawScreenButtonIcon (posX, posY, screen, hover)

	if screen == coordinateScreen then
		self:DrawScreenButtonIcon_Connection(posX, posY, hover)
	end

	if screen == characterScreen then
		self:DrawScreenButtonIcon_Character(posX, posY, hover)
	end

	if screen == deckScreen then
		self:DrawScreenButtonIcon_Deck(posX, posY, hover)
	end

end

-- =======================================================================

function BaseScreen:DrawScreenButtonIcon_Connection (posX, posY, hover)

	if hover then
		love.graphics.setColor(232, 232, 232, 232)
	else
		love.graphics.setColor(192, 192, 192, 128)
	end

	local size = 4
	love.graphics.setLineWidth(1)
	love.graphics.line(posX-size, posY-size, posX+size, posY+size)
	love.graphics.line(posX-size, posY+size, posX+size, posY-size)

end

-- =======================================================================

function BaseScreen:DrawScreenButtonIcon_Character (posX, posY, hover)

	local alpha = 64
	local color = 128

	if hover then

		alpha = 232
		color = 232

	end

	local radius = 4

	love.graphics.setColor(color, color, color, alpha)
	love.graphics.circle('fill', posX, posY, radius, 12)

	love.graphics.setLineWidth(1)
	love.graphics.setColor(232, 232, 232, 128)
	love.graphics.circle('line', posX, posY, radius, 12)

end

-- =======================================================================

function BaseScreen:DrawScreenButtonIcon_Deck (posX, posY, hover)

	local alpha = 64
	local color = 128

	if hover then

		alpha = 232
		color = 232

	end

	local size = 4

	love.graphics.setColor(color, color, color, alpha)
	love.graphics.rectangle('fill', posX-size, posY-size, 2*size, 2*size)

	love.graphics.setLineWidth(1)
	love.graphics.setColor(232, 232, 232, 128)
	love.graphics.rectangle('line', posX-size, posY-size, 2*size, 2*size)

end

-- =======================================================================

function BaseScreen:CheckScreenButtonPress (button, curX, curY)

	if button == MouseButton.Left then

		local s = self:GetHoveredScreenButton()
		if s then
			screen:Set(s)
		end

	end

end

-- =======================================================================

function BaseScreen:GetHoveredScreenButton()

	local curX, curY = mouse:GetCursorPos()
	for screen in self.screenButtonRenders:EachKey() do

		local screenButtonRender = self.screenButtonRenders[screen]
		if curX < screenButtonRender.width and math.abs(curY - screenButtonRender.posY) < screenButtonRender.border/2 then

			return screen

		end

	end

end

-- BACK BUTTON ===========================================================
-- =======================================================================

function BaseScreen:CreateBackButton()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local posX = windowSizeX * 0.225
	local posY = windowSizeY-64

	local backButton = loveFrames.Create('button')
	self.backButton = backButton

	backButton:SetSize(420, 32)
	backButton:SetPos(posX, posY, 'center')
	backButton:SetText('<<< BACK')
	backButton:SetFont('screen-button')

	backButton.OnClick = function()

		screen:Set(characterScreen)

	end

end

-- =======================================================================

function BaseScreen:RemoveBackButton()
	
	self.backButton:Remove()

end

-- =======================================================================

function BaseScreen:DrawBackButton()
	
	self.backButton:draw()

end

-- MENU ICON =============================================================
-- =======================================================================

function BaseScreen:DrawMenuIcon()

	if self:GetHoveredScreenButton() ~= nil then
		return
	end

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local posX = 12
	local posY = windowSizeY-18
	local size = 4

	self.menuIconRender = { posX = posX, posY = posY, size = size }
	local hover = self:IsMenuIconHovered()

	if hover then
		love.graphics.setColor(192, 192, 192, 192)
	else
		love.graphics.setColor(128, 128, 128, 128)
	end

	love.graphics.setLineWidth(1)
	love.graphics.line(posX-size, posY-size, posX+size, posY+size)
	love.graphics.line(posX-size, posY+size, posX+size, posY-size)

	if hover then

		love.graphics.setColor(192, 192, 192, 192)
		fonts:GetFont('screen-switchboard'):SetAsLoveFont()
		love.graphics.print("Back to Menu (!Quits Game)", 24, windowSizeY-24-1)

	end

end

-- =======================================================================

function BaseScreen:CheckMenuIconPress (button, curX, curY)

	if button == MouseButton.Left then

		if self:IsMenuIconHovered() then

			StateCleaner.new():Clean()
			screen:Set(menuScreen)

		end

	end

end

-- =======================================================================

function BaseScreen:IsMenuIconHovered()
	
	local menuIconRender = self.menuIconRender
	if not menuIconRender then
		return
	end

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local curX, curY = mouse:GetCursorPos()

	return math.abs(curX - menuIconRender.posX) < menuIconRender.size and math.abs(curY - menuIconRender.posY) < menuIconRender.size

end

-- POPUP =================================================================
-- =======================================================================

function BaseScreen:DrawPopup()

	if not self.eventPanel then
		return
	end

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setColor(0, 19, 38, 162)
	love.graphics.rectangle('fill', 0, 0, windowSizeX, windowSizeY)

	self:DrawBackGradient(12, 16, 12, 4, 2)
	self.eventPanel:Draw()

end

-- EVENT PANEL ===========================================================
-- =======================================================================

function BaseScreen:OpenEventPanel()

	self:Unhook()
	self.eventPanel = EventPanel.new()
	self.eventPanel:ShowNextEvent()

end

-- =======================================================================

function BaseScreen:CloseEventPanel()

	self:Hook()
	self.eventPanel = nil
	
	self:ReactToEventPanelClosed()

end

-- =======================================================================

-- [Screen classes can override this]
function BaseScreen:ReactToEventPanelClosed()
	
end

-- EXPLANATION BUTTON ====================================================
-- =======================================================================

function BaseScreen:DrawExplanationButton (posX)

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local render = self:DrawBottomButton(windowSizeX - 128 + (posX or 0), 48, self.explanationButtonRender, '?')
	self.explanationButtonRender = render

end

-- =======================================================================

function BaseScreen:CheckExplanationButtonPress (button, curX, curY)
	
	if self:IsBottomButtonHovered(self.explanationButtonRender) then
		self:ShowExplanation()
	end

end

-- EXPLANATION ===========================================================
-- =======================================================================

function BaseScreen:HasExplanation()

	return self.__static__.explanationID ~= nil

end

-- =======================================================================

function BaseScreen:GetExplanation()
	
	local ID = self.__static__.explanationID
	return explanations:GetExplanation(ID)

end

-- =======================================================================

function BaseScreen:ShowExplanation (explanation)
	
	explanation = explanation or self:GetExplanation()
	
	self.explanation = explanation
	explanation:Suppress()

end

-- =======================================================================

function BaseScreen:HideExplanation()
	
	self.explanation = nil

end

-- =======================================================================

function BaseScreen:DrawExplanation()
	
	if not self.explanation then
		return
	end

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setColor(0, 0, 32, 232 + 8)
	love.graphics.rectangle('fill', 0, 0, windowSizeX, windowSizeY)

	local font = fonts:GetFont('explanation-text')
	local fontHeight = font:GetHeight()

	local posY = 0
	local lineSpaceY = 12
	local totalLinesSizeY = 0
	local textSizeX = self.explanation:GetWidth()
	local lines = self.explanation:GetLines()
	local lineRenders = Collection.new()

	for line in lines:Each() do

		local _, l = font:CalculateSize(line, textSizeX)
		lineRenders:Add({ text = line, posY = posY })

		totalLinesSizeY = posY + l*fontHeight
		posY = totalLinesSizeY + lineSpaceY

	end

	local titleFont = fonts:GetFont('explanation-title')
	local titleFontHeight = titleFont:GetHeight()
	local titleSpaceY = 24

	local dismissLabelFont = fonts:GetFont('explanation-dismiss-label')
	local dismissLabelFontHeight = dismissLabelFont:GetHeight()
	local dismissLabelSpaceY = 24

	local backSizeX = textSizeX + 64
	local backSizeY = totalLinesSizeY + 128

	for n = 1, 4 do

		local alpha = 232 - n*48
		love.graphics.setColor(8, 32, 92, alpha)

		local sizeX = backSizeX + n * 24
		local sizeY = backSizeY + n * 18

		love.graphics.rectangle('fill', windowSizeX/2-sizeX/2, windowSizeY/2-sizeY/2, sizeX, sizeY)

	end

	local baseY = windowSizeY/2 - (totalLinesSizeY + titleSpaceY + titleFontHeight + dismissLabelSpaceY + dismissLabelFontHeight)/2

	titleFont:SetAsLoveFont()
	love.graphics.setColor(192, 192, 232, 232)
	love.graphics.printf(self.explanation:GetTitle(), 0, baseY, windowSizeX, 'center')
	baseY = baseY + titleSpaceY + titleFontHeight

	font:SetAsLoveFont()
	love.graphics.setColor(232, 232, 232)
	for lineRender in lineRenders:Each() do

		love.graphics.printf(lineRender.text, windowSizeX/2-textSizeX/2, baseY + lineRender.posY, textSizeX, 'center')

	end

	baseY = baseY + totalLinesSizeY + dismissLabelSpaceY
	dismissLabelFont:SetAsLoveFont()
	love.graphics.setColor(192, 192, 232, 232)
	love.graphics.printf("<Click to close>", 0, baseY, windowSizeX, 'center')

end

-- =======================================================================

function BaseScreen:ShowsExplanation()
	
	return self.explanation ~= nil

end

-- MOUSE =================================================================
-- =======================================================================

-- [Screen classes can override this]
function BaseScreen:OnMouseButtonPress (button, curX, curY)

end

-- ... ===================================================================
-- =======================================================================

function BaseScreen:GetMouseEvents()

	return self.mouseEvents

end

