
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

CharacterScreen = {}
CharacterScreen.__index = CharacterScreen

-- =======================================================================

setmetatable(CharacterScreen, BaseScreen)

-- =======================================================================

CharacterScreen.__static__ = 
{
	explanationID = 'characters'
}

-- =======================================================================

function CharacterScreen.new()

	local new = BaseScreen.new()
	setmetatable(new, CharacterScreen)

	new:Hook()

	return new

end

-- ENTER/LEAVE ===========================================================
-- =======================================================================

function CharacterScreen:Enter (context)

	self:CreateUI()
	self:OpenEventPanel()

end

-- =======================================================================

function CharacterScreen:Leave()

	if self.eventPanel then
		self.eventPanel:Close()
	end

	self:RemoveUI()

end

-- CREATE UI =============================================================
-- =======================================================================

function CharacterScreen:CreateUI()

	local buttonSizeX = 320
	local buttonSizeY = 24

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local spaceY = 232 - 16
	local baseY = windowSizeY - 180
	
	local baseX1 = 240
	local baseX2 = windowSizeX/2 + baseX1 - 60

	self.characters = Collection.new()
	for n, character in characters:GetCharacters():EachIndexed() do

		local buttons = Collection.new()
		local posY = baseY - (math.ceil(n/2)-1)*spaceY

		local baseX = pick(n % 2 == 1, baseX1, baseX2)

		-- 'Operations' button
		if #character:GetOperations() > 0 then

			local button = loveFrames.Create('button')
			button:SetText("[ Operations ]")
			buttons:Add(button)
			button.OnClick = function()

				screen:Set(operationScreen, { character = character })

			end

		end

		-- 'Chips' buttons
		if character:DealsItems(ItemClass.Chip) then

			local button = loveFrames.Create('button')
			button:SetText("[ Purchase chips ]")
			buttons:Add(button)
			button.OnClick = function()

				screen:Set(chipScreen, { character = character })
				
			end

		end

		-- 'Consoles' button
		if character:DealsItems(ItemClass.Console) then

			local button = loveFrames.Create('button')
			button:SetText("[ Purchase consoles ]")
			buttons:Add(button)
			button.OnClick = function()

				screen:Set(consoleScreen, { character = character })
				
			end

		end

		-- 'Coordinates' button
		if character:DealsItems(ItemClass.Coordinate) then

			local button = loveFrames.Create('button')
			button:SetText("[ Purchase coordinates ]")
			buttons:Add(button)
			button.OnClick = function()

				screen:Set(coordinateScreen, { character = character })
				
			end

		end

		-- 'Modules' button
		if character:DealsItems(ItemClass.Module) then

			local button = loveFrames.Create('button')
			button:SetText("[ Purchase modules ]")
			buttons:Add(button)
			button.OnClick = function()

				screen:Set(moduleScreen, { character = character })

			end

		end

		-- 'Data' button
		if character:AcceptsData() then

			local button = loveFrames.Create('button')
			button:SetText("[ Sell data ]")
			buttons:Add(button)
			button.OnClick = function()

				screen:Set(dataScreen, { character = character })

			end

		end

		self.characters:Add({ character = character, buttons = buttons, baseX = baseX, posY = posY })

		-- Button positions
		local buttonSpaceY = buttonSizeY + 4
		local startY = posY - (#buttons-1)*buttonSpaceY/2
		for n, button in buttons:EachIndexed() do

			button:SetFont('character-button')
			button:SetSize(buttonSizeX, buttonSizeY)
			button:SetPos(baseX + 256 + 16, startY + (n-1)*buttonSpaceY, true)

		end

	end

end

-- REMOVE UI =============================================================
-- =======================================================================

function CharacterScreen:RemoveUI()

	for character in self.characters:Each() do

		for button in character.buttons:Each() do
			button:Remove()
		end

	end

end	

-- DRAW ==================================================================
-- =======================================================================

function CharacterScreen:Draw()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	self:DrawBackTexture(1)
	self:DrawBackGradient(4, 24, 18, 8, 4)
	self:DrawMenuIcon()
	self:DrawScreenButtons()
	self:DrawCredits()
	self:DrawWaitButton()
	self:DrawExplanationButton(16)

	if #characters:GetCharacters() == 0 then

		love.graphics.setColor(192, 192, 192, 192)
		fonts:GetFont('screen-label'):SetAsLoveFont()
		love.graphics.printf('( You have no connections )', 0, windowSizeY-240, windowSizeX, 'center')

	end

	for character in self.characters:Each() do

		local radius = 64
		
		local avatarSizeX = 128
		local avatarSizeY = 128

		local sex = character.character:GetName():GetSex()
		local sprite = sprites:GetSprite(pick(sex == Sex.Male, 'character_man', 'character_woman'))
		local spriteSizeX = sprite:getWidth()
		local spriteSizeY = sprite:getHeight()

		love.graphics.setColor(8, 64, 168, 168)
		love.graphics.draw(sprite, character.baseX, character.posY, 0, avatarSizeX/spriteSizeX, avatarSizeY/spriteSizeY, spriteSizeX/2, spriteSizeY/2)

		love.graphics.setColor(128, 192, 255, 232)
		fonts:GetFont('character-name'):SetAsLoveFont()
		love.graphics.printf(character.character:GetName():GetNominative(), character.baseX-radius, character.posY+36, 2*radius, 'center')

		local lines = Collection.new()
		local posY = character.posY+radius
		
		local role = character.character:GetRole()
		if role then

			lines:Add({ text = strFormat('%s', role:GetText()) })

		end
		
		local affiliation = character.character:GetAffiliation()
		if affiliation then

			lines:Add({ text = strFormat('<%s>', affiliation:GetName()) })

		end

		for line in lines:Each() do
		
			posY = posY + 12
			love.graphics.setColor(128, 192, 255, 192)
			fonts:GetFont('character-role'):SetAsLoveFont()
			love.graphics.printf(line.text, character.baseX-120, posY, 240, 'center')
		
		end

		local relationship = character.character:GetRelationship()
		local status = relationship:GetStatus()

		status:GetColor():SetAsLoveColor()
		fonts:GetFont('character-relationship'):SetAsLoveFont()
		love.graphics.printf(strFormat('Relationship: %s', status:GetText()), character.baseX-120, posY+12, 240, 'center')

		for button in character.buttons:Each() do
			button:draw()
		end

		if #character.buttons == 0 then

			love.graphics.setColor(128, 192, 255, 192)
			fonts:GetFont('character-no-actions-label'):SetAsLoveFont()
			love.graphics.printf("(No actions available)", character.baseX + 256 + 16 - 120, posY - 48, 240, 'center')			

		end

	end

end

-- WAIT BUTTON ===========================================================
-- =======================================================================

function CharacterScreen:DrawWaitButton()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local render = self:DrawBottomButton(windowSizeX - 192 - 16, 128, self.waitButtonRender, "Wait")
	self.waitButtonRender = render

end

-- =======================================================================

function CharacterScreen:CheckWaitButtonPress()
	
	if self:IsBottomButtonHovered(self.waitButtonRender) then

		ticker:Tick()
		self:OpenEventPanel()

	end

end

-- EVENT PANEL ===========================================================
-- =======================================================================

-- [Base screen override]
function CharacterScreen:ReactToEventPanelClosed()
	
	self:RemoveUI()
	self:CreateUI()

end

-- MOUSE =================================================================
-- =======================================================================

function CharacterScreen:OnMouseButtonPress (button, curX, curY)

	self:CheckMenuIconPress(button, curX, curY)
	self:CheckScreenButtonPress(button, curX, curY)
	self:CheckExplanationButtonPress(button, curX, curY)
	self:CheckWaitButtonPress(button, curX, curY)

end

