
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

DeckScreen = {}
DeckScreen.__index = DeckScreen

-- =======================================================================

setmetatable(DeckScreen, BaseScreen)

-- =======================================================================

DeckScreen.__static__ = 
{
	explanationID = 'deck'	
}

-- =======================================================================

function DeckScreen.new()

	local new = BaseScreen.new()
	setmetatable(new, DeckScreen)

	new:Hook()

	return new

end

-- ENTER/LEAVE ===========================================================
-- =======================================================================

function DeckScreen:Enter (context)

	self:CreateUI()

end

-- =======================================================================

function DeckScreen:Leave()

	self:RemoveUI()

end

-- CREATE UI =============================================================
-- =======================================================================

function DeckScreen:CreateUI()

	self:CreateConsoleUI()
	self:CreateModuleUI()
	self:CreateChipUI()

end

-- =======================================================================

function DeckScreen:CreateConsoleUI()

	self.prevConsoleButton = loveFrames.Create('button')
	self.nextConsoleButton = loveFrames.Create('button')

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local distFromCenterX = 300
	local baseY = windowSizeY - 120

	local prevConsoleButton = self.prevConsoleButton
	prevConsoleButton:SetSize(80, 28)
	prevConsoleButton:SetPos(windowSizeX/2 - distFromCenterX, baseY, true)
	prevConsoleButton:SetText('<<<')

	local nextConsoleButton = self.nextConsoleButton
	nextConsoleButton:SetSize(80, 28)
	nextConsoleButton:SetPos(windowSizeX/2 + distFromCenterX, baseY, true)
	nextConsoleButton:SetText('>>>')

	prevConsoleButton.OnClick = function (button)
		self:MoveConsoleSelectionIndex(-1)
	end

	nextConsoleButton.OnClick = function (button)
		self:MoveConsoleSelectionIndex(1)
	end

end

-- =======================================================================

function DeckScreen:CreateModuleUI()
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local slots = deck:GetConsole():GetModuleSlots()

	local spaceX = 160
	local startX = (windowSizeX - (slots-1)*spaceX) / 2
	local posY = windowSizeY - 420

	local buttonSizeX = 145
	local buttonSizeY = 32

	self.moduleButtons = {}
	for index = 1, slots do

		local posX = startX + (index-1)*spaceX
		local button = loveFrames.Create('button')
		self.moduleButtons[index] = button

		button:SetSize(buttonSizeX, buttonSizeY)
		button:SetProperty('index', index)
		button:SetPos(posX, posY, true)
		button:SetFont('module-name')
		button:SetText('(...)')

		button.OnClick = function (button)

			local index = button:GetProperty('index')
			self:OpenModuleSelection(index)

		end

	end

end

-- =======================================================================

function DeckScreen:CreateChipUI()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local slots = deck:GetConsole():GetChipSlots()

	local baseY = windowSizeY - 620
	local stepY = 40

	local chips = assets:GetChips():Clone()
	for _, chip in deck:GetChips():EachNotBlank() do

		chips:RemoveValue(chip)

	end

	self.chipSelections = {}
	for index = 1, slots do

		local posY = baseY - stepY*index

		local selection = loveFrames.Create('multichoice')
		self.chipSelections[index] = selection

		selection:SetSize(640, 24)
		selection:SetPos(windowSizeX/2, posY, true)
		selection:SetProperty('index', index)

		local blankText = '--- Leave the chip slot blank ---'
		if deck:GetChips():HasItem(index) then

			local chip = chipEntities:Get(deck:GetChips():GetItem(index))
			local text = self:BuildChipLine(chip)

			selection:AddChoice(text, chip:GetID())
			selection:SetChoice(text)

		else

			selection:SetChoice(blankText)

		end

		selection:AddChoice(blankText, nil)
		for chip in chips:Each() do

			local chip = chipEntities:Get(chip)
			local text = "SWITCH TO: " .. self:BuildChipLine(chip) 

			selection:AddChoice(text, chip:GetID())

		end

		selection.OnChoiceSelected = function (selection, chip)

			local index = selection:GetProperty('index')
			if chip == nil then
				deck:GetChips():ClearItem(index)
			else
				deck:GetChips():SetItem(index, chip)
			end

			self:RemoveChipUI()
			self:CreateChipUI()

			deck:Rebuild()

		end

	end

end

-- =======================================================================

function DeckScreen:BuildChipLine (chip)

	return strFormat('[%s] %s', --[[chip:GetName()]] 'Chip', chip:GetDescription())

end

-- CONSOLE ===============================================================
-- =======================================================================

function DeckScreen:MoveConsoleSelectionIndex (delta)
	
	local index = assets:GetConsoles():Lookup(deck:GetConsole():GetID())
	assert(index, "Deck console was not located in the assets console list.")

	index = index + delta

	local newConsole = assets:GetConsoles()[index]
	if newConsole then

		deck:SetConsole(newConsole)

		self:RemoveModuleUI()
		self:CreateModuleUI()

		self:RemoveChipUI()
		self:CreateChipUI()

	end

end

-- MODULE SELECTION ======================================================
-- =======================================================================

function DeckScreen:OpenModuleSelection (index)

	self.selectedModule = index

end

-- =======================================================================

function DeckScreen:ClearModuleSelection()
	
	self.selectedModule = nil

end

-- REMOVE UI =============================================================
-- =======================================================================

function DeckScreen:RemoveUI()

	self.prevConsoleButton:Remove()
	self.nextConsoleButton:Remove()

	self:RemoveModuleUI()
	self:RemoveChipUI()

end	

-- =======================================================================

function DeckScreen:RemoveModuleUI()
	
	for _, moduleButton in ipairs(self.moduleButtons) do
		moduleButton:Remove()
	end

end

-- =======================================================================

function DeckScreen:RemoveChipUI()
	
	for _, chipSelection in ipairs(self.chipSelections) do
		chipSelection:Remove()
	end

end

-- DRAW ==================================================================
-- =======================================================================

function DeckScreen:Draw()

	self:DrawBackTexture(1)
	self:DrawBackGradient(4, 24, 18, 8, 4)
	self:DrawMenuIcon()
	self:DrawScreenButtons()
	self:DrawExplanationButton()
	self:DrawConsole()
	self:DrawConsoleButtons()
	self:DrawModules()
	self:DrawModuleDescription()
	self:DrawModuleSelection()
	self:DrawChips()

end

-- =======================================================================

function DeckScreen:DrawConsole()
	
	local console = deck:GetConsole()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local rectSizeX = 240
	love.graphics.setColor(16, 16, 16, 128)
	love.graphics.rectangle('fill', windowSizeX/2-rectSizeX, windowSizeY-160, 2*rectSizeX, 80)

	love.graphics.setLineWidth(1)
	love.graphics.setColor(64, 64, 92, 128)
	love.graphics.rectangle('line', windowSizeX/2-rectSizeX, windowSizeY-160, 2*rectSizeX, 80)

	love.graphics.setColor(232, 232, 232, 232)
	fonts:GetFont('deck-screen-label'):SetAsLoveFont()
	love.graphics.printf("[Console]", 0, windowSizeY-180, windowSizeX, 'center')

	local text = console:GetName()
	
	love.graphics.setColor(232, 232, 232)
	fonts:GetFont('console-name'):SetAsLoveFont()
	love.graphics.printf(text, windowSizeX/2-rectSizeX, windowSizeY-128, 2*rectSizeX, 'center')

	love.graphics.setColor(192, 192, 192, 232)
	local text = strFormat('%d Modules / %d Chips', console:GetModuleSlots(), console:GetChipSlots())
	love.graphics.printf(text, windowSizeX/2-rectSizeX, windowSizeY-72, 2*rectSizeX, 'center')

end

-- =======================================================================

function DeckScreen:DrawConsoleButtons()
	
	if #assets:GetConsoles() <= 1 then
		return
	end

	self.prevConsoleButton:draw()
	self.nextConsoleButton:draw()

end

-- =======================================================================

function DeckScreen:DrawModules()
	
	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local hover
	for index, moduleButton in ipairs(self.moduleButtons) do

		-- Text + Tokens
		if deck:GetModules():HasItem(index) then

			local module = Module:GetClass(deck:GetModules():GetItem(index))
			moduleButton:SetText(module:GetName():upper())

			local posX = moduleButton:GetX() + moduleButton:GetWidth()/2
			local posY = moduleButton:GetY() + moduleButton:GetHeight()/2
			self:DrawModuleTokens(module, posX, posY, Color.newFromRGB(64, 64, 232))

		else

			moduleButton:SetText('(select)')

		end

		-- Module button
		moduleButton:draw()

		-- Selected module
		if index == self.selectedModule then

			love.graphics.setLineWidth(3)
			love.graphics.setColor(128, 128, 232, 232)

			local border = 1
			local buttonSizeX = moduleButton:GetWidth()
			local buttonSizeY = moduleButton:GetHeight()

			love.graphics.rectangle('line', moduleButton:GetX()-border, moduleButton:GetY()-border, buttonSizeX+2*border-1, buttonSizeY+2*border)

		end

		if moduleButton:GetHover() then
			hover = true
		end

	end

	local text
	if self.selectedModule then

		text = "Select a module to install into the slot:"

	else

		text = pick(hover, "Click to replace with another module", "< ^^ Modules ^^ >")

	end

	love.graphics.setColor(232, 232, 232, 232)
	fonts:GetFont('deck-screen-label'):SetAsLoveFont()
	love.graphics.printf(text, 0, 580, windowSizeX, 'center')

end

-- =======================================================================

function DeckScreen:DrawModuleTokens (module, posX, posY, color)

	if not module:RequiresStaticTokens() then
		return
	end

	posY = posY + 16
	local spaceX = 18

	local tokens = module:GetStaticRequiredTokens()
	local startX = posX - (tokens-1)*spaceX/2

	for n = 1, tokens do

		local posX = startX + (n-1)*spaceX

		color:SetAsLoveColor()
		love.graphics.setLineWidth(1)
		love.graphics.circle('fill', posX, posY, 8, 6)
		love.graphics.circle('line', posX, posY, 8, 6)

	end

end

-- =======================================================================

function DeckScreen:DrawModuleSelection()
	
	self.moduleSelections = Map.new()
	if not self.selectedModule then
		return
	end

	local lineCounts = {}
	local count = #assets:GetModules()
	while count > 0 do

		local maxLineCount = 7
		local lineCount = math.min(count, maxLineCount)
		table.insert(lineCounts, lineCount)

		count = count - lineCount

	end

	local curX, curY = mouse:GetCursorPos()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local stepY = 60
	local baseY = windowSizeY - 320

	local moduleIndex = 1
	for lineIndex, lineCount in ipairs(lineCounts) do

		local posY = baseY + (lineIndex-1)*stepY

		local spaceX = 180
		local startX = (windowSizeX - (lineCount-1)*spaceX) / 2

		for index = 1, lineCount do

			local module = Module:GetClass(assets:GetModules()[moduleIndex])
			moduleIndex = moduleIndex + 1

			local posX = startX + (index-1)*spaceX

			self:DrawModuleTokens(module, posX, posY, Color.newFromRGB(64, 64, 232))

			local sizeX = 145
			local sizeY = 32

			local loaded = deck:GetModules():GetItemList():Contains(module:GetID())
			local hover = math.abs(posX-curX) < sizeX/2 and math.abs(posY-curY) < sizeY/2

			love.graphics.setColor(0, 0, 92, 232)
			love.graphics.rectangle('fill', posX-sizeX/2, posY-sizeY/2, sizeX, sizeY)

			love.graphics.setColor(64, 128, 232, pick(loaded, 64, pick(hover, 255, 192)))
			love.graphics.rectangle('line', posX-sizeX/2, posY-sizeY/2, sizeX, sizeY)

			local name = module:GetName()
			fonts:GetFont('module-name'):SetAsLoveFont()
			love.graphics.printf(name:upper(), posX-sizeX/2, posY-4, sizeX, 'center')

			self.moduleSelections[module] = { posX = posX, posY = posY, sizeX = sizeX, sizeY = sizeY }

		end

	end

end

-- =======================================================================

function DeckScreen:DrawModuleDescription()
	
	local module
	for index, moduleButton in ipairs(self.moduleButtons) do

		if moduleButton:GetHover() and deck:GetModules():HasItem(index) then
			module = Module:GetClass(deck:GetModules():GetItem(index))
		end

	end

	if not module then

		local curX, curY = love.mouse.getPosition()
		module = self:GetModuleSelectionOnPos(curX, curY)

	end

	if module then

		local description = module:GetDescription()
		local lines = select(2, description:gsub('\n', '\n')) + 1

		local windowSizeX = love.window.getWidth()
		local windowSizeY = love.window.getHeight()

		local sizeX = 620
		love.graphics.setColor(232, 232, 232)
		fonts:GetFont('module-description'):SetAsLoveFont()

		local baseY = windowSizeY-480 - (lines-1)*12
		love.graphics.printf(description, windowSizeX/2-sizeX, baseY, 2*sizeX, 'center')
	
		local traitLine = ''
		for trait in module:GetTraits():Each() do

			if traitLine:len() > 0 then
				traitLine = traitLine .. ' | '
			end

			traitLine = traitLine .. trait.label

		end

		fonts:GetFont('module-traits'):SetAsLoveFont()
		love.graphics.printf(traitLine, windowSizeX/2-sizeX, baseY-16, 2*sizeX, 'center')

	end

end

-- =======================================================================

function DeckScreen:DrawChips()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setColor(232, 232, 232, 232)
	fonts:GetFont('deck-screen-label'):SetAsLoveFont()
	love.graphics.printf("< ^^ Chips ^^ >", 0, 332, windowSizeX, 'center')

	for _, cs in ipairs(self.chipSelections) do

		cs:draw()
		if cs.haslist then

			cs.list:draw()

		end

	end

end

-- MOUSE =================================================================
-- =======================================================================

function DeckScreen:OnMouseButtonPress (button, curX, curY)

	self:CheckMenuIconPress(button, curX, curY)
	self:CheckScreenButtonPress(button, curX, curY)
	self:CheckExplanationButtonPress(button, curX, curY)

	if button == MouseButton.Right then
		self:ClearModuleSelection()
		return
	end

	local module = self:GetModuleSelectionOnPos(curX, curY)
	if module then

		deck:GetModules():SetItem(self.selectedModule, module:GetID())
		self:ClearModuleSelection()

	else
		self:ClearModuleSelection()
	end

end

-- =======================================================================

function DeckScreen:GetModuleSelectionOnPos(curX, curY)

	if not self.moduleSelections then
		return
	end

	for module, rect in self.moduleSelections:Each() do

		if math.abs(curX-rect.posX)<rect.sizeX/2 and math.abs(curY-rect.posY)<rect.sizeY/2 then
			return module
		end

	end

end
