
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

NetworkScreen = {}
NetworkScreen.__index = NetworkScreen

-- =======================================================================

setmetatable(NetworkScreen, BaseScreen)

-- =======================================================================

NetworkScreen.__static__ = 
{
	explanationID = 'network'
}

-- =======================================================================

function NetworkScreen.new()

	local new = BaseScreen.new()
	setmetatable(new, NetworkScreen)

	new:Hook()

	return new

end

-- ENTER/LEAVE ===========================================================
-- =======================================================================

function NetworkScreen:Enter (context)

	self.network = context.network
	self.operation = context.operation
	self.tutorial = context.tutorial

	local network = context.network
	self.run = Run.new(network)
	
	self.networkRender = NetworkRender.new(network)
	self.networkControl = NetworkControl.new(network)

	self.modulesRender = ModulesRender.new(self:GetRun():GetModules())
	self.tokensRender = TokensRender.new()
	self.traceRender = TraceRender.new()

	if self.operation then

		self.operationRender = OperationRender.new(self.operation)

	end

	if self.tutorial then

		self.tutorialRender = TutorialRender.new(self.tutorial)

	end

	local entrance = network:GetEntrance()
	self.networkControl:GetCameraControl():SetPos(entrance:GetPosX(), entrance:GetPosY()+40)
	network:ExtendAccess(entrance)
	network:ExtendVision(entrance)

	self.disconnected = nil

end

-- =======================================================================

function NetworkScreen:Leave()

	self.networkRender:Release()
	self.networkControl:Release()
	self.modulesRender:Release()

	if self.operationRender then

		self.operationRender:Release()
		self.operationRender = nil

	end

	if self.tutorialRender then

		self.tutorialRender:Release()
		self.tutorialRender = nil

	end

end

-- DRAW ==================================================================
-- =======================================================================

function NetworkScreen:Draw()

	self.networkRender:Draw()
	self.modulesRender:Draw()

	self.tokensRender:Draw()
	
	self:DrawHint()

	if self.network:RunsTrace() then

		self.traceRender:Draw(self.network:GetTrace())

	end

	if self.operationRender then

		self.operationRender:Draw()

	end

	if self.tutorialRender then

		self.tutorialRender:Draw()

	end

	if self:IsDisconnected() then

		self:DrawDisconnectPanel()

	else

		self:DrawDisconnectButton()

	end

end

-- =======================================================================

function NetworkScreen:DrawHint()

	if self.modulesRender:GetHoveredModuleRender() ~= nil then
		return
	end

	local hint = self:BuildHint()
	if hint then

		local xSize = 625
		skin:GetColor('network:hint'):SetAsLoveColor()

		local windowSizeX = love.window.getWidth()
		local windowSizeY = love.window.getHeight()

		local posY = windowSizeY - 48 - 8
		if self.network:RunsTrace() then

			posY = posY - 8

		end

		fonts:GetFont('network-hint'):SetAsLoveFont()
		love.graphics.printf(hint, windowSizeX/2 - xSize, posY, 2*xSize, 'center')

	end

end

-- =======================================================================

function NetworkScreen:DrawDisconnectButton()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local posX = windowSizeX-20
	local posY = windowSizeY-128-12
	local size = 4

	self.disconnectButton = { posX = posX, posY = posY, size = size }

	love.graphics.setLineWidth(1)
	skin:GetColor('network:disconnect-button'):SetAsLoveColor(pick(self:IsDisconnectButtonHovered(), 168, 128))
	love.graphics.line(posX-size, posY-size, posX+size, posY+size)
	love.graphics.line(posX-size, posY+size, posX+size, posY-size)

end

-- =======================================================================

function NetworkScreen:DrawDisconnectPanel()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setColor(8, 8, 32, 192)
	love.graphics.rectangle('fill', 0, 0, windowSizeX, windowSizeY)

	local posY = windowSizeY / 2

	local stepSizeY = 16
	local baseSizeY = 72

	for n = 8, 1, -1 do

		skin:GetColor('network:disconnect-back'):SetAsLoveColor(32 + 12 * n)
		love.graphics.rectangle('fill', 0, posY-baseSizeY/2-n*stepSizeY, windowSizeX, baseSizeY+2*n*stepSizeY)

	end

	local font = fonts:GetFont('network-disconnect')
	font:SetAsLoveFont()

	skin:GetColor('network:disconnect-label'):SetAsLoveColor()
	love.graphics.printf('[ Disconnected ]', 0, posY-font:GetHeight()/2, windowSizeX, 'center')

	fonts:GetFont('network-disconnect-dismiss'):SetAsLoveFont()
	skin:GetColor('network:disconnect-dismiss'):SetAsLoveColor()
	love.graphics.printf("Click to leave the network", windowSizeX/2 - 256, posY + 48, 512, 'center')

end

-- HINT ==================================================================
-- =======================================================================

function NetworkScreen:BuildHint()

	local mod = self.networkControl:GetActiveModule()
	if mod then

		local use = mod:GetUse()
		if use == ModuleUse.Node then

			local curX, curY = mouse:GetCursorPos()
			local node = self.networkRender:GetNodeOnScreenCoords(curX, curY)
			if node then

				return mod:GetHintForNode(node)

			else

				local verb = pick(mod:HasTrait(ModuleTrait.Coroutine), "install", "run")
				return strFormat("[Select a node to %s %s on]", verb, mod:GetName())

			end

		end

		if use == ModuleUse.Subroutine then

			local curX, curY = mouse:GetCursorPos()
			local subroutine = self.networkRender:GetSubroutineOnScreenCoords(curX, curY)
			if subroutine then

				return mod:GetHintForSubroutine(subroutine)

			else

				return strFormat("[Select a subroutine to run %s on]", mod:GetName())

			end

		end

	else

		local curX, curY = mouse:GetCursorPos()
		local node = self.networkRender:GetNodeOnScreenCoords(curX, curY)
		if node then

			local visibility = node:GetVisibility()
			if visibility == NodeVisibility.Full then
			
				return node:GetHint_Accessed()

			end

			if visibility == NodeVisibility.Class then

				return node:GetHint_Detected()

			end

			if visibility == NodeVisibility.Layout then

				return "(Unknown node)"

			end

		end

	end

	if self:IsDisconnectButtonHovered() then
		return "<Disconnect>"
	end

	if self.tokensRender:IsHovered() then

		local tokens = self.run:GetTokens():GetCount()
		return strFormat("< You have %d[T] >", tokens)

	end

end

-- TIME ==================================================================
-- =======================================================================

function NetworkScreen:ReceiveTimePulse (pulse)
	
	self.network:ReceiveTimePulse(pulse)
	self.networkRender:ReceiveTimePulse(pulse)
	self.run:ReceiveTimePulse(pulse)

	if self.operation then

		self.operation:ReceiveTimePulse(pulse)
		
	end

end

-- TUTORIAL ==============================================================
-- =======================================================================

function NetworkScreen:RunsTutorial()
	
	return self.tutorial ~= nil

end

-- =======================================================================

function NetworkScreen:ReceiveTutorialImpulse (impulse, context)

	if not self.tutorial then
		return
	end

	self.tutorial:React(impulse, Values.new(context))

end

-- =======================================================================

function NetworkScreen:RecreateModuleRenders()

	self.modulesRender = ModulesRender.new(self:GetRun():GetModules())

end

-- DISCONNECT ============================================================
-- =======================================================================

function NetworkScreen:Disconnect()

	self.disconnected = true

end

-- =======================================================================

function NetworkScreen:IsDisconnected()

	return self.disconnected ~= nil

end

-- ... ===================================================================
-- =======================================================================

function NetworkScreen:Exit()

	if self:RunsTutorial() then

		StateCleaner.new():Clean()
		screen:Set(tutorialsScreen)

	else

		screen:Set(characterScreen)

	end

end

-- MOUSE =================================================================
-- =======================================================================

function NetworkScreen:OnMouseButtonPress (button, curX, curY)

	if self:IsDisconnected() then

		self:Exit()
		return

	end

	if self:RunsTutorial() and self.tutorialRender:IsHovered() then

		local tutorial = self.tutorial
		local hint = tutorial:GetShownHint()

		if hint:HasNextHint() then

			tutorial:ShowHint(hint:GetNextHint())

		end

		return

	end

	self.networkRender:OnMouseButtonPress(button, curX, curY)

	if button == MouseButton.Left then
		if self.disconnectButton and self:IsDisconnectButtonHovered() then

			self.run:Terminate()
			self:Exit()

		end
	end

end

-- =======================================================================

function NetworkScreen:IsDisconnectButtonHovered()

	if not self.disconnectButton then
		return
	end

	local curX, curY = mouse:GetCursorPos()
	return math.abs(curX-self.disconnectButton.posX) < self.disconnectButton.size and math.abs(curY-self.disconnectButton.posY) < self.disconnectButton.size

end

-- EXPOSE ================================================================
-- =======================================================================

function NetworkScreen:GetRun()
	
	return self.run

end

-- =======================================================================

function NetworkScreen:GetNetwork()
	
	return self.network

end

-- =======================================================================

function NetworkScreen:GetNetworkRender()
	
	return self.networkRender

end

-- =======================================================================

function NetworkScreen:GetNetworkControl()
	
	return self.networkControl

end

-- =======================================================================

function NetworkScreen:GetDownloads()
	
	return self.downloads

end

