
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

OperationScreen = {}
OperationScreen.__index = OperationScreen

-- =======================================================================

setmetatable(OperationScreen, BaseScreen)

-- =======================================================================

OperationScreen.__static__ = 
{
	explanationID = 'operations'	
}

-- =======================================================================

function OperationScreen.new()

	local new = BaseScreen.new()
	setmetatable(new, OperationScreen)

	new:Hook()

	return new

end

-- ENTER/LEAVE ===========================================================
-- =======================================================================

function OperationScreen:Enter (context)

	self.character = context.character
	self:CreateUI()

end

-- =======================================================================

function OperationScreen:Leave()

	if self.eventPanel then
		self.eventPanel:Close()
	end

	self:RemoveUI()

end

-- CREATE UI =============================================================
-- =======================================================================

function OperationScreen:CreateUI()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	local baseY = windowSizeY - 180
	local spaceY = 272

	self.operations = Collection.new()
	for n, operation in self.character:GetOperations():EachIndexed() do

		local op = { operation = operation }
		self.operations:Add(op)

		op.posY = baseY - (n-1)*spaceY
		op.description = operation:GetDescription()

		local coordinate = operation:GetCoordinate()
		op.network = coordinate:GetNetworkName()
		op.rank = coordinate:GetRankText()

		if coordinate:IsExpired() then

			op.attempt = true

		else

			local button = loveFrames.Create('button')
			op.connectButton = button

			button:SetSize(120, 18)
			button:SetFont('connect-button')
			button:SetPos(windowSizeX/2, op.posY+16+12+12, true)
			button:SetText('Connect >>>')

			button.OnClick = function()

			 	local coordinate = operation:GetCoordinate()
				local networkBuilder = NetworkBuilder.new(coordinate)
				local network = networkBuilder:Build(operation)

				screen:Set(networkScreen, { network = network, operation = operation })
				coordinate:Expire()

				if operation:HasPayload() then

					local explanation = explanations:GetExplanation('payload')
					if not explanation:IsSuppressed() then

						screen:GetCurrent():ShowExplanation(explanation)
						
					end

				end

			end

		end

		-- Parts
		op.parts = Collection.new()

		local baseY = op.posY-92
		local spaceY = 16
		
		if operation:HasTargets() then

			for n, target in operation:GetTargets():EachIndexed() do

				local part = { target = target }
				op.parts:Add(part)

				part.posY = baseY - (n-1)*spaceY

				local class = target:GetClass()
				if class == OperationTargetClass.Data then

					local data = target:GetData()
					part.complete = assets:GetData():Contains(data) or nil

				end

				if class == OperationTargetClass.Coordinate then

					local coordinate = target:GetCoordinate()
					part.complete = assets:GetCoordinates():Contains(coordinate) or nil

				end

			end

		end

		if operation:HasPayload() then

			local part = { payload = operation:GetPayload(), posY = baseY }
			op.parts:Add(part)

			if part.payload:IsComplete() then

				part.complete = true

			end

		end

		-- Complete
		local isNotComplete = function (part) return part.complete == nil end
		if #op.parts:Select(isNotComplete) == 0 then

			op.complete = true

		end

		-- 'Complete' button
		if op.complete then

			local button = loveFrames.Create('button')
			op.completeButton = button

			button:SetSize(380, 28)
			button:SetPos(windowSizeX/2, op.posY+8, true)
			button:SetFont('operation-complete-button')
			button:SetText('> Complete <')
			button.customborder = { color = {21, 154, 41, 232}, width = 4 }

			button.OnClick = function()

				self.character:CompleteOperation(operation)

				self:RemoveUI()
				self:CreateUI()

				self:OpenEventPanel()

			end

		end

		-- 'Abandon' button
		if op.attempt and (not op.complete) then

			local button = loveFrames.Create('button')
			op.abandonButton = button

			button:SetSize(380, 24)
			button:SetPos(windowSizeX/2, op.posY+8, true)
			button:SetFont('operation-complete-button')
			button:SetText('Abandon')
			button.customborder = { color = {92, 32, 32, 232}, width = 3 }

			button.OnClick = function()

				self.character:AbandonOperation(operation)

				self:RemoveUI()
				self:CreateUI()

				self:OpenEventPanel()

			end

		end

	end

	self:CreateBackButton()

end

-- REMOVE UI =============================================================
-- =======================================================================

function OperationScreen:RemoveUI()

	for operation in self.operations:Each() do

		if operation.connectButton then
			operation.connectButton:Remove()
		end

		if operation.completeButton then
			operation.completeButton:Remove()
		end

		if operation.abandonButton then
			operation.abandonButton:Remove()
		end

	end

	self:RemoveBackButton()

end	

-- DRAW ==================================================================
-- =======================================================================

function OperationScreen:Draw()

	local windowSizeX = love.window.getWidth()
	local windowSizeY = love.window.getHeight()

	love.graphics.setColor(232, 232, 232)
	fonts:GetFont('screen-label'):SetAsLoveFont()
	love.graphics.printf(strFormat("ACCEPTED OPERATIONS (%s)", self.character:GetName():GetNominative()), 0, windowSizeY-80, windowSizeX, 'center')

	self:DrawBackTexture(1)
	self:DrawBackGradient(4, 24, 18, 8, 4)
	self:DrawMenuIcon()
	self:DrawScreenButtons()
	self:DrawBackButton()
	self:DrawExplanationButton()
	self:DrawCredits()

	if #self.operations == 0 then

		love.graphics.setColor(192, 192, 192, 192)
		fonts:GetFont('screen-text'):SetAsLoveFont()
		love.graphics.printf('( No operations )', 0, windowSizeY-240, windowSizeX, 'center')

	end

	for operation in self.operations:Each() do

		local op = operation.operation
		local posY = operation.posY

		local sizeX = 1120
		local sizeY = 232

		-- Back
		love.graphics.setLineWidth(1)
		love.graphics.setColor(128, 128, 232, 32)
		love.graphics.rectangle('fill', (windowSizeX-sizeX)/2, posY-sizeY+60+12, sizeX, sizeY)

		-- Baseline
		if operation.attempt then

			if operation.complete then

				operation.completeButton:draw()

			else

				love.graphics.setColor(232, 232, 232, 128)
				love.graphics.printf('(Operation Failed)', windowSizeX/2-320, posY+16+8, 2*320, 'center')

			end

		else

			-- Coordinate
			love.graphics.setColor(232, 232, 232)
			fonts:GetFont('coordinate-name'):SetAsLoveFont()
			love.graphics.printf(strFormat('[ %s ]', operation.network), 0, posY-8, windowSizeX, 'center')

			love.graphics.setColor(232, 232, 232)
			fonts:GetFont('coordinate-rank'):SetAsLoveFont()
			love.graphics.printf(strFormat("< Expected security: %s >", operation.rank), 0, posY-8+16, windowSizeX, 'center')

			local lineOuterSpaceX = 240
			local lineInnerSpaceX = operation.network:len()*4 + 64
			local linePosY = posY

			love.graphics.setLineWidth(1)
			love.graphics.setColor(232, 232, 232, 64)
			love.graphics.line(lineOuterSpaceX, linePosY, windowSizeX/2-lineInnerSpaceX, linePosY)
			love.graphics.line(windowSizeX/2+lineInnerSpaceX, linePosY, windowSizeX-lineOuterSpaceX, linePosY)

			operation.connectButton:draw()

		end

		-- Description
		fonts:GetFont('operation-description'):SetAsLoveFont()
		love.graphics.setColor(232, 232, 232, 192)
		love.graphics.printf(operation.description, windowSizeX/2-560, posY-48, 2*560, 'center')

		-- Parts
		for part in operation.parts:Each() do

			local text = ''
			if part.target then

				local target = part.target
				local class = target.class

				if class == OperationTargetClass.Data then

					text = "Download Data"

				end

				if class == OperationTargetClass.Coordinate then

					text = "Secure a coordinate"

				end

			end

			if part.payload then

				local payload = part.payload
				local condition = payload:GetCondition()

				text = strFormat('%s', condition:GetText())

			end

			local color 
			if operation.attempt then

				if part.complete then
					color = Color.newFromRGB(102, 236, 87)
				else
					color = Color.newFromRGB(192, 64, 64)
				end

			else
				color = Color.newFromRGB(232, 232, 232, 192)
			end

			color:SetAsLoveColor()
			love.graphics.printf(strFormat('[ %s ]', text), windowSizeX/2-320, part.posY, 2*320, 'center')

		end

		-- Abandon
		if (not operation.attempt) or operation.complete then

			local abandonPosX = windowSizeX/2 + sizeX/2 - 16
			local abandonPosY = operation.posY - sizeY + 64 + 12 + 8 + 4

			local curX, curY = mouse:GetCursorPos()
			local hover = (self:GetAbandonIconOnPos(curX, curY) == operation)

			local size = pick(hover, 6, 4)
			love.graphics.setLineWidth(1)
			love.graphics.setColor(128, 128, 128, pick(hover, 128, 64))
			love.graphics.line(abandonPosX-size, abandonPosY-size, abandonPosX+size, abandonPosY+size)
			love.graphics.line(abandonPosX-size, abandonPosY+size, abandonPosX+size, abandonPosY-size)

			operation.abandonIcon = { posX = abandonPosX, posY = abandonPosY, size = size }	

			if hover then

				fonts:GetFont('operation-abandon-icon'):SetAsLoveFont()
				love.graphics.printf('Abandon', abandonPosX-size-128-8, abandonPosY-6, 128, 'right')

			end

		else

			operation.abandonButton:draw()

		end

	end

end

-- EVENT PANEL ===========================================================
-- =======================================================================

-- [Base screen override]
function OperationScreen:ReactToEventPanelClosed()

	if #self.character:GetOperations() == 0 then

		screen:Set(characterScreen)
		return

	end

	self:RemoveUI()
	self:CreateUI()

end

-- MOUSE =================================================================
-- =======================================================================

function OperationScreen:OnMouseButtonPress (button, curX, curY)

	self:CheckMenuIconPress(button, curX, curY)
	self:CheckScreenButtonPress(button, curX, curY)
	self:CheckExplanationButtonPress(button, curX, curY)

	if button == MouseButton.Left then

		local operation = self:GetAbandonIconOnPos(curX, curY)
		if operation then

			self.character:AbandonOperation(operation.operation)

			self:RemoveUI()
			self:CreateUI()

			self:OpenEventPanel()

		end

	end

end

-- =======================================================================

function OperationScreen:GetAbandonIconOnPos (screenX, screenY)

	for operation in self.operations:Each() do
		if operation.abandonIcon then

			local abandonIcon = operation.abandonIcon
			if math.abs(screenX-abandonIcon.posX) < abandonIcon.size and math.abs(screenY-abandonIcon.posY) < abandonIcon.size then

				return operation

			end

		end
	end

end

