
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Collection = {}

-- =======================================================================

function Collection.new (values)

    local new = values or {}
    setmetatable(new, Collection)

    return new

end

-- DISPATCH ==============================================================
-- =======================================================================

function Collection.__index (collection, index)

	return Collection[index]

end

-- METHODS ===============================================================
-- =======================================================================

function Collection:Add (value)

	table.insert(self, value)

end 

-- =======================================================================

function Collection:Append (collection)

	for v in collection:Each() do
		table.insert(self, v)
	end

end

-- =======================================================================

function Collection:Each()
	
	local n = 0
	return function()
		n = n + 1
		return self[n]
	end

end

-- =======================================================================

function Collection:ReverseEach()

	local n = #self + 1
	return function()
		n = n - 1
		return self[n]
	end

end

-- =======================================================================

function Collection:EachIndexed()

	return ipairs(self)

end

-- =======================================================================

function Collection:ReverseEachIndexed()
	
	local n = #self + 1
	return function()
		n = n - 1
		if n <= 0 then
			return nil    
		end
        
		return n, self[n]
	end

end

-- =======================================================================

function Collection:Select (pred)

	local result = Collection.new()
	for _, v in ipairs(self) do
		if pred(v) then
			result:Add(v)
		end
	end

	return result

end

-- =======================================================================

function Collection:First (pred)

	if pred == nil then
		return assert(self[1], "Cannot call First() on an empty collection.")
	end

	for _, v in ipairs(self) do
		if pred(v) then
			return v
		end
	end

end

-- =======================================================================

function Collection:Last (pred)

	if pred == nil then
		return assert(self[#self], "Cannot call Last() on an empty collection.")
	end

	for n = #self, 1, -1 do
		if pred(self[n]) then
			return self[n]
		end
	end

end

-- =======================================================================

function Collection:Lookup (value)
	
	for n, v in ipairs(self) do
		if v == value then
			return n
		end
	end

end

-- =======================================================================

function Collection:LookupWithPredicate (pred)
	
	for n, v in ipairs(self) do
		if pred(v) then
			return n
		end
	end

end

-- =======================================================================

function Collection:Contains (value)
	
	return self:Lookup(value) ~= nil

end

-- =======================================================================

function Collection:Random()

	return self[random:IntValue(1, #self)]

end

-- =======================================================================

function Collection:PopFirst()

	local v = self[1]
	table.remove(self, 1)

	return v

end

-- =======================================================================

function Collection:PopLast()

	local v = self[#self]
	table.remove(self, #self)

	return v

end

-- =======================================================================

function Collection:Clone()
	
	local clone = {unpack(self)}
	setmetatable(clone, Collection)

	return clone

end

-- =======================================================================

function Collection:Map (map)

	local result = Collection.new()
	for _, v in ipairs(self) do
		result:Add(map(v))
	end

	return result

end

-- =======================================================================

function Collection:RemoveIndex (index)
	
	table.remove(self, index)

end

-- =======================================================================

function Collection:RemoveValue (value)

	local n = assert(self:Lookup(value), "Cannot remove value that is not present in the collection.")
	table.remove(self, n)

end

-- =======================================================================

function Collection:RemoveWithPredicate (pred)
	
	local n = 1
	while n <= #self do

		if pred(self[n]) then
			table.remove(self, n)
		else
			n = n + 1
		end

	end

end

-- =======================================================================

function Collection:Sort (comp)
	
	table.sort(self, comp)
	return self

end

-- =======================================================================

function Collection:Reverse()
	
	local size = #self
	for n = 1, math.floor(size / 2) do
		self[n], self[size-n+1] = self[size-n+1], self[n]
	end

	return self

end

-- =======================================================================

function Collection:Clear()
	
	while #self > 0 do
		table.remove(self, #self)
	end

end
