
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

function isNumber (val)
	return type(val) == 'number'
end

function isStr (val)
	return type(val) == 'string'
end

function isTable (val)
	return type(val) == 'table'
end

-- TABLE OPERATIONS ======================================================
-- =======================================================================

-- Returns an index on which the provided 
-- value is stored in the provided table.
function table.lookup (t, val)

	for k, v in pairs(t) do
		if v == val then
			return k
		end
	end

end

-- =======================================================================

-- Returns a boolean value based on whether 
-- the provided table contains the provided value.
function table.contains (t, val)
	
	return table.lookup(t, val) ~= nil
	
end

-- =======================================================================

function table.shallowClone (t)
	
	local result = {}
	for k, v in pairs(t) do
		result[k] = v
	end

	return result

end

-- =======================================================================

function table.deepClone (t)
	
	local result = {}
	for k, v in pairs(t) do
		if isTable(v) then
			result[k] = table.deepClone(v)
		else
			result[k] = v
		end
	end

end

-- =======================================================================

-- Returns a tabke which contains exactly those key-value pairs present 
-- in the provided table where the value meets the provided predicate.
function table.select (t, pred)

	local result = {}
	for k, v in pairs(t) do
		if pred(v) then
			result[k] = v
		end
	end
	
	return result

end

-- =======================================================================

-- Returns a sequence which contains all the values 
-- in the provided sequence that meet the provided predicate.
function table.iselect (t, pred)

	local result = {}
	for _, v in ipairs(t) do
		if pred(v) then
			table.insert(result, v)
		end
	end
	
	return result

end

-- =======================================================================

-- Returns a value in the provided 
-- table that meets the provided predicate.
function table.one (t, pred)

	for _, v in pairs(t) do
		if pred(v) then
			return v
		end
	end

end

-- =======================================================================

-- Returns how many values in the 
-- provided table meet the provided predicate.
function table.count (t, pred)

	local count = 0
	for _, v in pairs(t) do
		if pred(v) then
			count = count + 1
		end
	end
	
	return count

end

-- =======================================================================

-- Removes all the values in the table that meet the provided predicate. 
function table.removeWithPredicate (t, pred)

	local k, v
	while true do
		k, v = next(t, k)
		if k == nil then
			break
		end
		if pred(v) then
			t[k] = nil
		end
	end

end

-- =======================================================================

-- Removes a value contained in the table.
function table.removeValue (t, val)

	for k, v in pairs(t) do
		if v == val then
			table.remove(t, k)
			return
		end
	end

end

-- =======================================================================

-- Returns a random element from the provided sequence.
function table.random(t)

	return t[random:IntValue(1, #t)]

end

-- =======================================================================

-- Returns the first element in a sequence, and removes it from the sequence.
function table.popFirst (t)

	local v = t[1]
	table.remove(t, 1)
	
	return v

end

-- =======================================================================

-- Returns the last element in a sequence, and removes it from the sequence.
function table.popLast (t)

	local v = t[#t]
	table.remove(t, #t)
	
	return v

end

-- =======================================================================

function table.empty (t)

	return next(t) == nil 

end

-- ARITHMETIC ============================================================
-- =======================================================================

function math.round (n)

	if n % 1 < 0.5 then
		return math.floor(n)
	else
		return math.ceil(n)
	end

end

-- =======================================================================

function math.clamp (n, min, max)
	
	return pick(n > max, max, pick(n < min, min, n))

end

-- STRING ================================================================
-- =======================================================================

function string.explode (s, delim)

 	local tokens = {}
	for token in s:gmatch(strFormat('[^%s]+', delim)) do
		table.insert(tokens, token)
	end

	return tokens

end

-- =======================================================================

function string.trim (s)
  
	s = s:gsub('^%s+', ''):gsub('%s+$', '')
	return s

end

-- =======================================================================

function string.capitaliseFirst (s)
	
	s = s:sub(1, 1):upper() .. s:sub(2)
	return s

end

-- =======================================================================

function string.startsWith (s, start)

	return s:sub(1, #start) == start

end

-- CLASSES ===============================================================
-- =======================================================================

function isClass (instance, class)

	return getmetatable(instance) == class

end

-- SHORTCUTS =============================================================
-- =======================================================================

toStr = tostring
strFormat = string.format

-- ... ===================================================================
-- =======================================================================

function pick (condition, a, b)

	if condition then 
		return a
	end
	return b

end

