
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Module = Entity.new()
Module.__index = Module

-- =======================================================================

function Module:__class_setup__ (params)

	self.name = params.name
	self.use = params.use
	self.description = params.description
	self.tokens = params.tokens
	self.minNodeVisibility = params.minNodeVisibility or NodeVisibility.Full
	self.maxNodeVisibility = params.maxNodeVisibility
	self.traits = Collection.new(params.traits)
	self.staticTokens = params.staticTokens

	if params.nodeClassesLimit then
		self.nodeClassesLimit = Collection.new(params.nodeClassesLimit)
	end

	self.reactions = Map.new()
	self.handlers = Map.new()

end

-- =======================================================================

function Module:new (network)

	local new = {}
	setmetatable(new, self)

	new.network = network
	new.values = Values.new()
	new.stashes = Map.new()

	new:Init()

	return new

end

-- CONSTRUCTION ==========================================================
-- =======================================================================

-- [Module classes can override this]
function Module:Init()
	
end

-- EXECUTION =============================================================
-- =======================================================================

function Module:Execute()

	self:SpendRequiredTokens()
	self:Run()

end

-- =======================================================================

function Module:ExecuteOnNode (node)

	self:SpendRequiredTokens()
	self:RunOnNode(node)

	-- Exploit
	if self:HasTrait(ModuleTrait.Exploit) then

		node:RemoveExploit()

	end

	-- Subroutine reactions
	if node:GetClass() == NodeClass.Router then

		if not self:HasTrait(ModuleTrait.Undetectable) then

			node:ReceiveSubroutineImpulse(SubroutineImpulse.ModuleUse, { module = self })

		end

	end

	-- Tutorial reactions
	networkScreen:ReceiveTutorialImpulse(TutorialImpulse.ModuleExecution, { module = self })

end

-- =======================================================================

function Module:ExecuteOnSubroutine (subroutine)
	
	self:SpendRequiredTokens()
	self:RunOnSubroutine(subroutine)

end

-- CHECKS ================================================================
-- =======================================================================

function Module:CanExecute()

	if self:IsLocked() then
		return
	end

	if self:RequiresTokens() then

		local tokens = networkScreen:GetRun():GetTokens()
		if not tokens:CanSpend(self:GetRequiredTokens()) then
 
			return
			
		end

	end

	return true

end

-- =======================================================================

function Module:CanExecuteOnNode (node)

	-- Node class
	if self.nodeClassesLimit then

		if not self.nodeClassesLimit:Contains(node:GetClass()) then
			return
		end

	end

	-- Vision
	local visibility = node:GetVisibility()
	if self.minNodeVisibility and visibility < self.minNodeVisibility then
		return
	end

	if self.maxNodeVisibility and visibility > self.maxNodeVisibility then
		return
	end

	-- Subtle: Alert
	if self:HasTrait(ModuleTrait.Subtle) and node:IsAlerted() then
		return
	end

	-- Coroutine/Subtle: Trace
	if self:HasTrait(ModuleTrait.Subtle) or self:HasTrait(ModuleTrait.Coroutine) then

		if self.network:RunsTrace() and self.network:GetTrace():ContainsNode(node) then
			return
		end

	end

	-- Exploit
	if self:HasTrait(ModuleTrait.Exploit) and (not node:HasExploit()) then
		return
	end

	-- Module
	return self:AcceptsNode(node)

end

-- =======================================================================

-- [Module classes can override this]
function Module:AcceptsNode (node)

	return true

end

-- =======================================================================

function Module:CanExecuteOnSubroutine (subroutine)
	
	return self:AcceptsSubroutine(subroutine)

end

-- =======================================================================

-- [Module classes can override this]
function Module:AcceptsSubroutine (subroutine)

	return true

end

-- TRAITS ================================================================
-- =======================================================================

function Module:HasTrait (trait)
	
	return self.traits:Contains(trait)

end

-- =======================================================================

function Module:GetTraits()
	
	return self.traits

end

-- TOKENS ================================================================
-- =======================================================================

-- [Module classes can override this]
function Module:GetRequiredTokens()

	return self.tokens

end

-- =======================================================================

function Module:SpendRequiredTokens()
	
	if self:RequiresTokens() then

		local tokens = networkScreen:GetRun():GetTokens()
		tokens:Remove(self:GetRequiredTokens())

	end	

end

-- =======================================================================

function Module:RequiresTokens()

	return self:GetRequiredTokens() ~= nil

end

-- =======================================================================

function Module:GetStaticRequiredTokens()

	return self.staticTokens or self:GetRequiredTokens()

end

-- =======================================================================

function Module:RequiresStaticTokens()

	return self:GetStaticRequiredTokens() ~= nil

end

-- TOKENS: MODIFICATIONS =================================================
-- =======================================================================

function Module:SetRequiredTokens (tokens)

	self.tokens = tokens

end

-- =======================================================================

function Module:IncreaseRequiredTokens (inc)

	local tokens = self:GetRequiredTokens()
	self:SetRequiredTokens(tokens + inc)

end

-- =======================================================================

function Module:ReduceRequiredTokens (reduction, minimum)

	local tokens = self:GetRequiredTokens()
	self:SetRequiredTokens(math.max(tokens - reduction, minimum or 0))

end

-- LOCK ==================================================================
-- =======================================================================

function Module:Lock (cost)
	
	if self:IsLocked() then

		self.unlockCost = math.max(self.unlockCost, cost)

	else

		self.lock = true
		self.unlockCost = cost

	end

end

-- =======================================================================

function Module:Unlock()

	self.lock = nil
	self.unlockCost = nil

end

-- =======================================================================

function Module:IsLocked()
	
	return self.lock ~= nil

end

-- =======================================================================

function Module:GetUnlockCost()

	return self.unlockCost
	
end

-- REACTIONS =============================================================
-- =======================================================================

function Module:React (impulse)

	local reaction = self.reactions[impulse]
	if reaction then
	
		reaction(self)
		
	end

end

-- UTILITIES =============================================================
-- =======================================================================

function Module:CreateCoroutine (node, ID)
	
	local coroutine = Coroutine:NewInstance(ID)
	coroutine:Install(node)

end

-- DESCRIPTION ===========================================================
-- =======================================================================

function Module:GetDescription()
	
	return self.description

end

-- =======================================================================

-- [Module classes can override this]
function Module:GetRuntimeDescription()
	
	return self.description

end

-- EFFECTS ===============================================================
-- =======================================================================

function Module:CreateEffect (effect)
	
	local e = self:NewEffect(effect)

	local networkRender = networkScreen:GetNetworkRender()
	networkRender:AddEffect(e)

end

-- =======================================================================

function Module:CreateEffectOnNode (effect, node)
	
	local e = self:NewEffect(effect, node)
	
	local nodeRender = networkScreen:GetNetworkRender():GetNodeRender(node)
	nodeRender:AddEffect(e)

end

-- =======================================================================

function Module:CreateEffectOnSubroutine (effect, subroutine)
	
	local e = self:NewEffect(effect, subroutine)
	
	local host = subroutine:GetHost()
	local routerRender = networkScreen:GetNetworkRender():GetNodeRender(host)
	local subroutineRender = routerRender:GetSubroutineRender(subroutine)

	subroutineRender:AddEffect(e)

end

-- =======================================================================

function Module:NewEffect (effect, pivot)
	
	local e = Effect:NewInstance(effect, pivot)
	e.onEventRaised:Subscribe(self)

	return e

end

-- =======================================================================

function Module:OnEffectEventRaised (effect, event)
	
	local handler = self.handlers[event]
	if handler then

		handler(self, effect)

	end

end

-- HINT ==================================================================
-- =======================================================================

function Module:GetHintForNode (node)

	if self:HasTrait(ModuleTrait.Subtle) then

		if node:GetClass() == NodeClass.Router and node:IsAlerted() then

			return "(Subtle modules cannot run on Alerted nodes)"

		end

		local network = self.network
		if network:RunsTrace() then

			local trace = network:GetTrace()
			if trace:ContainsNode(node) then

				return "(Subtle modules cannot run on nodes that are under a Trace)"

			end

		end

	end

	if self:HasTrait(ModuleTrait.Exploit) then

		if node:GetClass() ~= NodeClass.Router or (not node:HasExploit()) then

			return "(Exploit modules have to be run on a node that contains an Exploit)"

		end

	end

	if self:CanExecuteOnNode(node) then

		local verb = pick(self:HasTrait(ModuleTrait.Coroutine), "install", "run")
		return strFormat("(Click to %s %s on this node)", verb, self.name)

	else

		local verb = pick(self:HasTrait(ModuleTrait.Coroutine), "be installed", "run")
		return strFormat("(%s cannot %s on this node)", self.name, verb)

	end

end

-- =======================================================================

function Module:GetHintForSubroutine (subroutine)

	if self:CanExecuteOnSubroutine(subroutine) then

		return strFormat("(Click to run %s on this subroutine)", self.name)

	else

		return strFormat("(%s cannot run on this subroutine)", self.name)

	end

end

-- EXPOSE ================================================================
-- =======================================================================

function Module:GetID()
	
	return self.ID

end

-- =======================================================================

function Module:GetName()
	
	return self.name

end

-- =======================================================================

function Module:GetUse()
	
	return self.use

end

