
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

Router = {}
Router.__index = Router

-- =======================================================================

setmetatable(Router, Obstacle)

-- =======================================================================

function Router.new (plan)

	local new = Obstacle.new(plan)
	setmetatable(new, Router)
			
	new.alerts = 0
	new.subroutines = Collection.new()
	new.coroutines = Collection.new()

	new:LoadFromPlan(plan)
	
	return new

end

-- CONSTRUCTION ==========================================================
-- =======================================================================

function Router:LoadFromPlan (plan)

	-- Perimeter
	self.perimeter = plan:GetPerimeter()

	-- Subroutines
	for subroutinePlan in plan:GetSubroutinePlans():Each() do

		local ID = subroutinePlan:GetID()
		local stealth = subroutinePlan:GetStealth()
		local values = subroutinePlan:GetValues()

		local subroutine = Subroutine:NewInstance(ID, values, stealth)
		self:HostSubroutine(subroutine)

	end

	-- Exploit
	if plan:HasExploit() then

		local exploitPlan = plan:GetExploitPlan()

		local ID = exploitPlan:GetID()
		local exposure = exploitPlan:GetExposure()
		local values = exploitPlan:GetValues()
		
		local exposure = ExposureCondition:NewInstance(exposure)
		local exploit = Exploit:NewInstance(ID, exposure, values)

		self:SetExploit(exploit)

	end

end

-- ACCESS ================================================================
-- =======================================================================

-- [Base Node override]
function Router:ReceiveAccess()

	for trait in deck:GetDetectedSubroutineTraits():Each() do

		self:DetectSubroutineTrait(trait)
		
	end

end

-- BREACH ================================================================
-- =======================================================================

function Router:Breach (context)

	-- Prevent
	if not context:HasImpact() then

		if self:CanSubroutinesBlock(BlockableAction.Breach) then
			return
		end

	end
	
	-- Perimeter
	local breach = true
	if self:HasPerimeter() then

		if context:CircumventsPerimeter() then

			self:ClearPerimeter()

		else

			self:ReducePerimeter()
			breach = nil

			networkScreen:ReceiveTutorialImpulse(TutorialImpulse.PerimeterReduction, { node = self })

		end

	end

	-- Subroutine reactions
	if not context:IsUndetectable() then

		self:ReceiveSubroutineImpulse(SubroutineImpulse.Breach)

	end

	if not breach then
		return
	end

	self:AllowAccess()

	-- Mark reactions
	self:ReceiveMarkImpulse(MarkImpulse.Breach)

	-- Exposition reactions
	self:ReceiveExposureImpulse(ExposureImpulse.Breach)
	for node in self:GetConnectedNodes(NodeClass.Router):Each() do

		node:ReceiveExposureImpulse(ExposureImpulse.BreachConnected)

	end

	-- Evasion reactions
	self.network:ReceiveEvasionImpulse(EvasionImpulse.Breach)

	-- Tutorial reactions
	networkScreen:ReceiveTutorialImpulse(TutorialImpulse.Breach, { node = self })

end

-- PERIMETER =============================================================
-- =======================================================================

function Router:HasPerimeter()
	
	return self.perimeter ~= nil

end

-- =======================================================================

function Router:GetPerimeter()
	
	return self.perimeter

end

-- =======================================================================

function Router:ReducePerimeter()
	
	self.perimeter = self.perimeter - 1

	if self.perimeter == 0 then
		self.perimeter = nil
	end

end

-- =======================================================================

function Router:ClearPerimeter()
	
	self.perimeter = 0

end

-- SUBROUTINES ===========================================================
-- =======================================================================

function Router:HostSubroutine (subroutine)

	self.subroutines:Add(subroutine)
	subroutine.host = self

end

-- =======================================================================

function Router:ReceiveSubroutineImpulse (impulse, context)
	
	for subroutine in self:GetActiveSubroutines():Each() do
		subroutine:React(impulse, context)
	end

end

-- =======================================================================

function Router:CanSubroutinesBlock (action)

	for subroutine in self:GetActiveSubroutines():Each() do

		if subroutine:CheckBlock(action) then
			return true
		end

	end

end

-- =======================================================================

function Router:DetectSubroutineTrait (trait)
	
	for subroutine in self.subroutines:Each() do
		subroutine:DetectTrait(trait)
	end

end

-- =======================================================================

function Router:GetActiveSubroutines()

	local isActive = function (s) return s:IsActive() end
	return self.subroutines:Select(isActive)

end

-- =======================================================================

function Router:GetSubroutines()

	return self.subroutines

end

-- COROUTINES ============================================================
-- =======================================================================

function Router:HostCoroutine (coroutine)
	
	self.coroutines:Add(coroutine)
	coroutine.host = self

	self:ReceiveSubroutineImpulse(SubroutineImpulse.Coroutine)

end

-- =======================================================================

function Router:GetActiveCoroutines()
	
	local isActive = function (c) return c:IsActive() end
	return self.coroutines:Select(isActive)

end

-- =======================================================================

function Router:GetCoroutines()
	
	return self.coroutines

end

-- VISION ================================================================
-- =======================================================================

-- [Base Node override]
function Router:StartsAsVisionBlock()

	return self:CanSubroutinesBlock(BlockableAction.Vision)

end

-- =======================================================================

function Router:CheckVisionBlock()
	
	if not self:CanSubroutinesBlock(BlockableAction.Vision) then

		self:AllowVision()

	end

end

-- TRACE =================================================================
-- =======================================================================

-- [Base Node override]
function Router:StartTracePoint (point)
	
	point:StartAdvance()

	self:ReceiveMarkImpulse(MarkImpulse.TraceHit)
	self:ReceiveSubroutineImpulse(SubroutineImpulse.Trace)

	for coroutine in self:GetActiveCoroutines():Each() do

		coroutine:Remove()

		-- Tutorial reactions
		networkScreen:ReceiveTutorialImpulse(TutorialImpulse.CoroutineTerminated, { coroutine = coroutine })

	end

end

-- =======================================================================

function Router:GetTraceSpeed()
	
	local base = 0.125
	local speed = base

	local speedModPerAlert = deck:GetTraceSpeedModPerAlert()
	speed = speed * (1 + speedModPerAlert * self.alerts)

	return speed

end

-- ALERTS ================================================================
-- =======================================================================

function Router:Alert (count)

	self.alerts = self.alerts + (count or 1)
	self:ReceiveSubroutineImpulse(SubroutineImpulse.Alert)

end

-- =======================================================================

function Router:IsAlerted()

	return self.alerts > 0

end

-- =======================================================================

function Router:GetAlertCount()

	return self.alerts

end

-- =======================================================================

function Router:ClearAllAlerts()
	
	self.alerts = 0

end

-- =======================================================================

function Router:ClearAlerts (count)

	self.alerts = math.max(self.alerts-count, 0)

end

-- =======================================================================

function Router:ClearAlert()

	self:ClearAlerts(1)

end

-- EXPLOIT ===============================================================
-- =======================================================================

function Router:SetExploit (exploit)
	
	self.exploit = exploit
	exploit.host = self

end

-- =======================================================================

function Router:RemoveExploit()
	
	self.exploit = nil

end

-- =======================================================================

function Router:HasExploit()
	
	return self.exploit ~= nil

end

-- =======================================================================

function Router:GetExploit()
	
	return self.exploit

end

-- EXPLOIT EXPOSITION ====================================================
-- =======================================================================

function Router:ReceiveExposureImpulse (impulse)
	
	if self:HasExploit() then

		self.exploit:ReceiveExposureImpulse(impulse)

	end

end

-- HINT ==================================================================
-- =======================================================================

-- [Base Node override]
function Router:GetHint_Accessed()

	local hint = Node.GetHint_Accessed(self)

	if self:HasExploit() then

		hint = hint .. '\n' 
		if self.exploit:IsExposed() then

			hint = hint .. "<Exploitable: R-click to exploit>"

		else

			hint = hint .. "<Exploitable>"

		end

	end

	return hint

end
