
-- (c) 2015 Martin Ziegler
-- martin.ziegler@outlook.cz

RouterRender = {}
RouterRender.__index = RouterRender

-- =======================================================================

setmetatable(RouterRender, NodeRender)

-- =======================================================================

function RouterRender.new (node)

	local new = NodeRender.new(node)
	setmetatable(new, RouterRender)

	new.subroutineRenders = Collection.new()
	new:CreateSubroutineRenders()

	return new

end

-- =======================================================================

function RouterRender:CreateSubroutineRenders()

	for subroutine in self.node:GetSubroutines():Each() do

		local subroutineRender = SubroutineRender.new(subroutine)
		self.subroutineRenders:Add(subroutineRender)

	end

end

-- =======================================================================

function RouterRender:Draw (screenX, screenY)

	local node = self.node

	self:SetupRenderColor()
	local rectSize = 20

	if not node:IsAccessed() then

		love.graphics.setLineWidth(1)
		love.graphics.rectangle('line', screenX-rectSize, screenY-rectSize, 2*rectSize, 2*rectSize)
		return

	end

	-- Perimeter
	if node:HasPerimeter() then

		local perimeterSizeStep = 12
		local perimeterSizeBase = 64

		love.graphics.setLineWidth(3)
		skin:GetColor('network:node-details'):SetAsLoveColor()

		for n = 1, self.node:GetPerimeter() do

			local size = perimeterSizeBase + (n-1)*perimeterSizeStep
			love.graphics.polygon('line', screenX, screenY-size/2, screenX+size/2, screenY, screenX, screenY+size/2, screenX-size/2, screenY)

		end

	end

	-- Rect
	self:SetupRenderColor()
	love.graphics.rectangle('fill', screenX-rectSize, screenY-rectSize, 2*rectSize, 2*rectSize)

	-- Block
	if node:BlocksAccess() then

		local space = 5
		love.graphics.setLineWidth(4)
		skin:GetColor('network:node-details'):SetAsLoveColor()

		love.graphics.line(screenX-rectSize-space, screenY-rectSize, screenX-rectSize-space, screenY+rectSize)
		love.graphics.line(screenX+rectSize+space, screenY-rectSize, screenX+rectSize+space, screenY+rectSize)
		love.graphics.line(screenX-rectSize, screenY-rectSize-space, screenX+rectSize, screenY-rectSize-space)
		love.graphics.line(screenX-rectSize, screenY+rectSize+space, screenX+rectSize, screenY+rectSize+space)

	end

	-- Trace
	local trace = networkScreen.network:GetTrace()
	if trace and trace:ContainsNode(node) then

		local tracePoint = trace:GetPoint(node)
		local size = tracePoint.advance * rectSize

		skin:GetColor('network:trace-node'):SetAsLoveColor()
		love.graphics.rectangle('fill', screenX-size, screenY-size, 2*size, 2*size)

	end

	self:DrawSubroutineRenders(screenX, screenY)
	self:DrawCoroutineRenders(screenX, screenY)
	self:DrawAlerts(screenX, screenY)
	self:DrawExploit(screenX, screenY)	

end

-- =======================================================================

function RouterRender:DrawExploit (screenX, screenY)

	if not self.node:HasExploit() then
		return
	end

	local hover = self:IsHovered()
	local exploit = self.node:GetExploit()
	local exposed = exploit:IsExposed()

	local colorFill
	local colorOutline

	if exposed then

		if hover then

			colorFill = 'network:exploit-exposed-hover'
			colorOutline = 'network:exploit-exposed-hover-outline'

		else

			colorFill = 'network:exploit-exposed'
			colorOutline = 'network:exploit-exposed-outline'

		end

	else

		colorFill = 'network:exploit'
		colorOutline = 'network:exploit-outline'

	end

	local radius = pick(exposed, 10, 12)

	skin:GetColor(colorFill):SetAsLoveColor()
	love.graphics.circle('fill', screenX, screenY, radius, pick(exposed, 24, 8))
	
	love.graphics.setLineWidth(2)
	skin:GetColor(colorOutline):SetAsLoveColor()
	love.graphics.circle('line', screenX, screenY, radius, pick(exposed, 24, 8))

	if not exposed then

		radius = radius - 8
		love.graphics.setLineWidth(2)
		skin:GetColor(colorOutline):SetAsLoveColor()

		love.graphics.line(screenX-radius, screenY-radius, screenX+radius, screenY+radius)
		love.graphics.line(screenX-radius, screenY+radius, screenX+radius, screenY-radius)

	end

	if hover then

		if exposed then

			local text = strFormat("<EXPLOIT: %s>", exploit:GetDescription())
			self:DrawDescription(text, skin:GetColor('network:exploit-description-outline'), 420)

		else

			local condition = exploit:GetCondition()
			local text = strFormat("<EXPLOIT: %s to expose>", condition:GetDescription())

			self:DrawDescription(text, skin:GetColor('network:exploit-description-outline'), 420)

		end

	end

end

-- =======================================================================

function RouterRender:DrawSubroutineRenders (screenX, screenY)

	local space = 16

	local activeRenders = self.subroutineRenders:Select(function (render) return render:GetSubroutine():IsActive() end)
	local startX = screenX - ( (#activeRenders-1)*space )/2

	local iconY = screenY - 32 - 2
	if self.node:BlocksAccess() then
		iconY = iconY - 4
	end

	for n, subroutineRender in activeRenders:EachIndexed() do

		local iconX = startX + (n-1)*space
		subroutineRender:Draw(iconX, iconY)

	end

end

-- =======================================================================

function RouterRender:DrawCoroutineRenders (screenX, screenY)

	local space = 16

	local activeCoroutines = self.node:GetActiveCoroutines()
	local startY = screenY - ( (#activeCoroutines-1)*space )/2
	
	for n, coroutine in activeCoroutines:EachIndexed() do

		local iconX = screenX - 32 - 4
		if self.node:BlocksAccess() then
			iconX = iconX - 4
		end

		local iconY = startY + (n-1)*space

		local coroutineRender = CoroutineRender.new(coroutine)
		coroutineRender:Draw(iconX, iconY)

	end

end

-- =======================================================================

function RouterRender:DrawAlerts (screenX, screenY)
	
	local spaceX = 12
	local baseX = screenX + 32

	if self.node:BlocksAccess() then
		baseX = baseX + 6
	end

	local maxPerLine = 5
	local count = self.node:GetAlertCount()

	local line = 1
	while count > 0 do

		local countInLine = math.min(count, maxPerLine)
		self:DrawAlertLine(baseX + (line - 1)*spaceX, screenY, countInLine)

		count = count - countInLine
		line = line + 1

	end

end

-- =======================================================================

function RouterRender:DrawAlertLine (screenX, screenY, count)

	local spaceY = 12
	local startY = screenY - ((count - 1)*spaceY) / 2

	for n = 1, count do

		local size = 8
		local posY = startY + (n - 1)*spaceY

		love.graphics.setColor(232, 64, 64, 255)
		skin:GetColor('network:alert'):SetAsLoveColor()
		love.graphics.rectangle('fill', screenX - size / 2, posY - size / 2, size, size)

		love.graphics.setLineWidth(1)
		skin:GetColor('network:alert-outline'):SetAsLoveColor()
		love.graphics.rectangle('line', screenX - size / 2, posY - size / 2, size, size)

	end

end

-- TIME ==================================================================
-- =======================================================================

function RouterRender:ReceiveTimePulse (pulse)
	
	NodeRender.ReceiveTimePulse(self, pulse)

	for sr in self.subroutineRenders:Each() do
		sr:ReceiveTimePulse(pulse)
	end

end

-- CONTROL ===============================================================
-- =======================================================================

-- [Base Node control override]
function RouterRender:OnMouseButtonRelease (button, curX, curY)
	
	if self.node:HasExploit() and self.node:GetExploit():IsExposed() and button == MouseButton.Right  then

		-- ===
		-- TODO: Move this to Router
		-- ===

		networkScreen:ReceiveTutorialImpulse(TutorialImpulse.ExploitExploited, { exploit = self.node:GetExploit() })

		self.node:GetExploit():Execute()
		self.node:RemoveExploit()

	end

end

-- EXPOSE ================================================================
-- =======================================================================

function RouterRender:GetSubroutineRenders()
	
	return self.subroutineRenders

end

-- =======================================================================

function RouterRender:GetSubroutineRender (subroutine)

	return self.subroutineRenders:First(function (sr) return sr:GetSubroutine() == subroutine end)

end

